/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <QtGui/QPainter>
#include <QtCore/QRect>

#include "../corefactory.h"
#include "../fsm_I.h"
#include "../core_I.h"
#include "items.h"

StateItem::StateItem(int width, int height, Q3Canvas* can) : Q3CanvasEllipse(width, height, can){
  name = "";
  id = 0;
}

StateItem::~StateItem() {
  hide();
}

QString StateItem::text() {
  return name;
}

void StateItem::setText(QString s) {
  int len = s.length();
  if(len < 9) {
    name = s;
  } else {
    s.remove(8,len-8);
    name = s;
  }
}

void StateItem::drawShape(QPainter &painter) {
  Q3CanvasEllipse::drawShape(painter);
  painter.setPen(Qt::black);
  painter.drawText(boundingRect(),Qt::AlignCenter,text());
  if(isActive()) {
    painter.setBrush(Qt::black);
    painter.drawRect((int)x() - (int)(width()/2) - 2,(int)y() - 2,4,4);
    painter.drawRect((int)x() + (int)(width()/2) - 2,(int)y() - 2,4,4);
    painter.drawRect((int)x() - 2,(int)y() - (int)(height()/2) - 2,4,4);
    painter.drawRect((int)x() - 2,(int)y() + (int)(height()/2) - 2,4,4);
  }
}

QRect StateItem::boundingRect() const {
  return QRect((int)x() - 45,(int)y()- 45, width() + 35, height() + 35);
}

void StateItem::modifyPos(double posx, double posy) {
   Core_I& c = CoreFactory::getCore_I();
   Fsm_I& f = c.getCurrentFsm();
   f.modifyState(id, posx, posy);
}

Transline::Transline(Q3Canvas* can) : Q3CanvasLine(can) {
  id = 0;
}

Transline::~Transline() {
  hide();
}

void Transline::drawShape(QPainter &painter) {
  if(isActive()) {
    painter.setPen(Qt::red);
  }
  Q3CanvasLine::drawShape(painter);
}

Leveltransfer::Leveltransfer(int width, int height, Q3Canvas* can) : Q3CanvasEllipse(width, height, can) {
  transid = 0;
  id = 0;
}

Leveltransfer::~Leveltransfer() {
  hide();
}

void Leveltransfer::drawShape(QPainter &painter) {
  Q3CanvasEllipse::drawShape(painter); 
  if(isActive()) { 
    painter.setPen(Qt::black);
    painter.setBrush(Qt::black);
    painter.drawRect((int)x() - (int)(width()/2) - 1,(int)y() - 1,2,2);
    painter.drawRect((int)x() + (int)(width()/2) - 1,(int)y() - 1,2,2);
    painter.drawRect((int)x() - 1,(int)y() - (int)(height()/2) - 1,2,2);
    painter.drawRect((int)x() - 1,(int)y() + (int)(height()/2) - 1,2,2);
  }
}

void Leveltransfer::modifyPos(double posx, double posy) {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  f.modifyTrans(transid, id, posx, posy);
}

LinkItem::LinkItem(int width, int height, Q3Canvas* can) : Q3CanvasEllipse(width, height, can) {
  id = 0;
}

LinkItem::~LinkItem(){
  hide(); 
}

void LinkItem::drawShape(QPainter &painter) {
  Q3CanvasEllipse::drawShape(painter);
  if(isActive()) { 
    painter.setPen(Qt::black);
    painter.setBrush(Qt::black);
    painter.drawRect((int)x() - (int)(width()/2) - 1,(int)y() - 1,2,2);
    painter.drawRect((int)x() + (int)(width()/2) - 1,(int)y() - 1,2,2);
    painter.drawRect((int)x() - 1,(int)y() - (int)(height()/2) - 1,2,2);
    painter.drawRect((int)x() - 1,(int)y() + (int)(height()/2) - 1,2,2);
  }  
}

void LinkItem::modifyPos(double posx, double posy) {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  f.modifyLink(id, posx, posy);

}

Linkentrance::Linkentrance(Q3Canvas* can) : Q3CanvasRectangle(can){
  transid = 0;
  id = 0;
}

Linkentrance::~Linkentrance() {
  hide();
}

void Linkentrance::drawShape(QPainter &painter) {
  if(isActive()) {
    painter.setPen(Qt::red);
  }
  Q3CanvasRectangle::drawShape(painter);
}

void Linkentrance::modifyPos(double posx, double posy) {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  f.modifyTrans(transid, id, posx, posy);

}

Joinpoint::Joinpoint(Q3Canvas* can) : Q3CanvasRectangle(can){
  transid = 0;
  id = 0;
}

Joinpoint::~Joinpoint() {
  hide();
}

void Joinpoint::drawShape(QPainter &painter) {
  if(isActive()) {
    painter.setPen(Qt::red);
  }
  Q3CanvasRectangle::drawShape(painter);
}

void Joinpoint::modifyPos(double posx, double posy) {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  f.getTransbyID(transid);
  f.getTransTrackpointbyID(id);
  int goal = f.getTransEnd();
  double jposx = f.getTransTrackpointPosx();
  double jposy = f.getTransTrackpointPosy();
  if(f.getFirstTrans()) {
    do {
      if (f.getTransEnd() == goal) {
	if(f.getTransFirstTrackpoint()) {
	  do {
	    if(f.getTransTrackpointJoin()) {
	      if((f.getTransTrackpointPosx() == jposx) && (f.getTransTrackpointPosy() == jposy)) {
		int curtransid = f.getTransID();
		int curid = f.getTransTrackpointID();
		f.modifyTrans(curtransid, curid, posx, posy);
		while(f.getTransNextTrackpoint()) {}
	      }
	    }
	  } while (f.getTransNextTrackpoint());
	}	
      }
    } while (f.getNextTrans());
  }
}

TrackpointItem::TrackpointItem(Q3Canvas* can) : Q3CanvasRectangle(can){
  transid = 0;
  id = 0;
}

TrackpointItem::~TrackpointItem() {
  hide();
}

void TrackpointItem::drawShape(QPainter &painter) {
  if(isActive()) {
    painter.setPen(Qt::red);   
    painter.setBrush(Qt::red);
  } 
  Q3CanvasRectangle::drawShape(painter);
}

void TrackpointItem::modifyPos(double posx, double posy) {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  f.modifyTrans(transid, id, posx, posy);
}

Hypertransition::Hypertransition(int width, int height, Q3Canvas* can) : Q3CanvasEllipse(width, height, can) {
  id = 0;
}

Hypertransition::~Hypertransition() {
  hide(); 
}

void Hypertransition::drawShape(QPainter &painter) {
  Q3CanvasEllipse::drawShape(painter);
  if(isActive()) { 
    painter.setPen(Qt::black);
    painter.setBrush(Qt::black);
    painter.drawRect((int)x() - (int)(width()/2) - 1,(int)y() - 1,2,2);
    painter.drawRect((int)x() + (int)(width()/2) - 1,(int)y() - 1,2,2);
    painter.drawRect((int)x() - 1,(int)y() - (int)(height()/2) - 1,2,2);
    painter.drawRect((int)x() - 1,(int)y() + (int)(height()/2) - 1,2,2);
  }
}

void Hypertransition::modifyPos(double posx, double posy) {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  f.modifyHypertrans(id, posx, posy);
}

Hierarchyexit::Hierarchyexit(Q3Canvas* can) : Q3CanvasRectangle(can){
  transid = 0;
  id = 0;
}

Hierarchyexit::~Hierarchyexit() {
  hide();
}

void Hierarchyexit::drawShape(QPainter &painter) {
  if(isActive()) {
    painter.setPen(Qt::red);
  }
  Q3CanvasRectangle::drawShape(painter);
}

void Hierarchyexit::modifyPos(double posx, double posy) {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  f.modifyTrans(transid, id, posx, posy);
}


TranslineText::TranslineText(QString qs, Q3Canvas* can) : Q3CanvasText(qs, can) {
  id = 0;
  name = "Transname";
}



QString TranslineText::text() {
  return name;
}

void TranslineText::setText(QString s) {

}

void TranslineText::draw(QPainter &painter) { 
  Q3CanvasText::draw(painter);
}

QRect TranslineText::boundingRect() const {
  return QRect((int)x(),(int)y(), 20 , 20);
}

void TranslineText::modifyPos(double posx, double posy) {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  f.getTransbyID(id);
  int level = f.getTransLevel();
  f.modifyTrans(id, posx, posy, level);
}
