/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <QtGui/QMenu>
#include <QtGui/QContextMenuEvent>

#include "projecttreewidget.h"

#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"


ProjectTreeWidget::ProjectTreeWidget(QWidget* w) : QTreeWidget(w) {
  createActions();
}

void ProjectTreeWidget::setQWidget(QWidget* w){
  qw = w;
}

void ProjectTreeWidget::contextMenuEvent(QContextMenuEvent* e) {
   QMenu contextMenu(this);
   contextMenu.addAction(renameproject);
   contextMenu.addAction(addfsm);
   contextMenu.addSeparator();  
   contextMenu.addAction(renamefsm);
   contextMenu.addAction(newhierarchylevel);
   contextMenu.addAction(deletefsm);   
   contextMenu.exec(e->globalPos());
   
}

void ProjectTreeWidget::mouseDoubleClickEvent(QMouseEvent* e) {
  if(e->button() == Qt::LeftButton) {
    if(this->currentItem()) {
      QTreeWidgetItem* it = this->currentItem();
      QString qs = it->text(0);
      string  s = QStrToStr(qs); 
      Core_I& c = CoreFactory::getCore_I(); 
      Fsm_I& fbefore = c.getCurrentFsm();
      int idbefore = fbefore.getFsmID();
      bool found = false;
      int id = 0;
      if(!(strcmp(s.c_str(),c.getProjectName().c_str()) == 0)) {
	if(c.getFirstFsm()) {
	  do {
	    Fsm_I& f = c.getCurrentFsm();
	    if(strcmp(s.c_str(),f.getFsmName().c_str()) == 0) {
	      found = true;
	      id = f.getFsmID();
	      dynamic_cast<MainguiWindow*>(qw)->setcurrentfsmid(id);
	      if(id != idbefore) {
		dynamic_cast<MainguiWindow*>(qw)->setcurrentnumberofhlevels(0);
	      }
	      dynamic_cast<MainguiWindow*>(qw)->setcurrenthierarchylevel(0);
	      dynamic_cast<MainguiWindow*>(qw)->refresh();
	    }
	  } while (c.getNextFsm() && !found);
	}
	if(found) {
	  c.getFsm(id);
	} else {
	  bool go = true;
	  int i = 1;
	  while (go) {
	    string name = "level "; 
	    name = name + itos(i);
	    if(strcmp(s.c_str(),name.c_str()) == 0) {
	      go = false; 
	      dynamic_cast<MainguiWindow*>(qw)->setcurrenthierarchylevel(i);
	      QTreeWidgetItem* wit = it->parent();
	      QString qstr = wit->text(0);
	      string str = QStrToStr(qstr);
	      found = false;
	      if(c.getFirstFsm()) {
		do {
		  Fsm_I& f = c.getCurrentFsm();
		  if(strcmp(str.c_str(),f.getFsmName().c_str()) == 0) {
		    found = true;
		    id = f.getFsmID();
		    dynamic_cast<MainguiWindow*>(qw)->setcurrentfsmid(id);
		    dynamic_cast<MainguiWindow*>(qw)->refresh();
		    if(id != idbefore) {
		      dynamic_cast<MainguiWindow*>(qw)->setcurrentnumberofhlevels(0);
		    }
		  }
		} while (c.getNextFsm() && !found);
	      }
	      c.getFsm(id);
	    }
	  i++;
	  }
	}
      }
    }
  }
}

void ProjectTreeWidget::createActions(){
  addfsm = new QAction(this);
  addfsm->setObjectName(QString::fromUtf8("addFsm"));
  addfsm->setEnabled(true);    
  addfsm->setText(QApplication::translate("ProjectTreeWidget", "New FSM", 0, QApplication::UnicodeUTF8));
  newhierarchylevel = new QAction(this);
  newhierarchylevel->setObjectName(QString::fromUtf8("newHierarchyLevel"));
  newhierarchylevel->setEnabled(true);    
  newhierarchylevel->setText(QApplication::translate("ProjectTreeWidget", "Insert new level", 0, QApplication::UnicodeUTF8));
  deletefsm = new QAction(this);
  deletefsm->setObjectName(QString::fromUtf8("deleteFsm"));
  deletefsm->setEnabled(true);    
  deletefsm->setText(QApplication::translate("ProjectTreeWidget", "Delete FSM", 0, QApplication::UnicodeUTF8));
  renameproject = new QAction(this);
  renameproject->setObjectName(QString::fromUtf8("renameProject"));
  renameproject->setEnabled(true);    
  renameproject->setText(QApplication::translate("ProjectTreeWidget", "Rename project", 0, QApplication::UnicodeUTF8));
  renamefsm = new QAction(this);
  renamefsm->setObjectName(QString::fromUtf8("renameFsm"));
  renamefsm->setEnabled(true);    
  renamefsm->setText(QApplication::translate("ProjectTreeWidget", "Rename FSM", 0, QApplication::UnicodeUTF8));
}

string ProjectTreeWidget::QStrToStr(QString q) {
  string s (q.toAscii());
  return s;
}

string ProjectTreeWidget::itos(int i) {
  stringstream s;
  s << i;
  return s.str();
}
