/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <Qt3Support/Q3Canvas>
#include <Qt3Support/Q3CanvasView>
#include <QtGui/QMenu>
#include <QtGui/QContextMenuEvent>
#include <QtGui/QClipboard>
#include <QtCore/QTextStream>
#include <QtCore/QPoint>
#include <QtGui/QMatrix>
#include <cstdlib>
#include <iostream>
#include <cmath>

#include "view.h"
#include "mainguiwindow.h"
#include "items.h"
#include "newlink.h"
#include "newhyper.h"
#include "changestart.h"
#include "changeend.h"
#include "linkconnector.h"
#include "hierarchyconnector.h"
#include "../fsm_I.h"
#include "../core_I.h"
#include "../corefactory.h"

using namespace std;

View::View (Q3Canvas* c, QWidget* w) : Q3CanvasView (c,w){
  wid = w;
  can = c;
  initvariables();
  bgGrid = new QPixmap(":/icons/grid1.png",0,0);
  createActionsforContextMenu();
  connectContextMenuActions();
  can->update();
}

View::~View () {
  delete can;
}

void View::initvariables() {
  paintmode = false;
  mode = CHOOSE;
  lock = UNLOCKED;
  activatedItem = 0;
  newItem = 0;
  front = 0;
  back = 0;
  helpvar = 0;
  posx = 0.1;
  posy = 0.1;
  startx = 0.1;
  starty = 0.1;
  endx = 0.1;
  endy = 0.1;
  transbegin = -1;
  transend = -1;
  mousemoved = false;
  usegrid = false;
  transbegin = true;
  transinserted = false;
  foundstate = false;
  statecolor = Qt::red;
}

void View::contentsMousePressEvent(QMouseEvent* e) {
  //make sure the view window gets the focus, so property changes etc are saved
  setFocus(Qt::MouseFocusReason);

 if (paintmode) {
  posx = e->pos().x();
  posy = e->pos().y();
  mousemoved = false;
  Q3CanvasItemList cil = canvas()->collisions(e->pos());
  if(e->button() == Qt::RightButton) {
    if(cil.empty()) {
      activatedItem = 0;
    } else {
      activateItem(*cil.begin());
    }
  }
  canvas()->update();
  if(e->button() == Qt::LeftButton) {
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    switch(mode) {
    case STATE: 
      {
	(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	transbegin = true;
	addState();
	newItem->move(e->pos().x(),e->pos().y());
	placeItem(newItem);
	newItem = 0;
	//cout << "Draw State" << endl;
	if(lock == UNLOCKED) {
	  mode = CHOOSE;
	}
      }
      break;
    case TRANS:	
      {
	activateItem(0);
	if(transbegin) {
	  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	  if(cil.isEmpty()) { 
	    transstart = -1;
	    transend = -1;
	    addTrans();
	    f.modifyTrans(helpvar , (double) e->pos().x() + 5, (double) e->pos().y() + 5, getCurrentLevel());
	    addTrackpoint();
	    newItem->move(e->pos().x() - 2.5, e->pos().y() - 2.5);
	    placeItem(newItem);
	    newItem = 0;
	    activateItem(0);
	    startx = e->pos().x();
	    starty = e->pos().y();
	    transbegin = false;
	  } else {
	    transbegin = false;
	    Q3CanvasItem* i = cil.takeFirst();
	    if(i->rtti() == StateItem::RTTI) {
	      transstart = (dynamic_cast<StateItem*>(i))->id;
	      transend = -1;
	      foundstate = true;
	      f.getStatebyID(transstart);
	      startx = f.getStatePosx();
	      starty = f.getStatePosy();
	    } else if(i->rtti() == Joinpoint::RTTI) {
	      helpvar = (dynamic_cast<Joinpoint*>(i))->transid;
	      f.getTransbyID(helpvar); 
	      transstart = f.getTransStart(); 
	      int z;
	      z = (dynamic_cast<Joinpoint*>(i))->id;
	      f.getTransTrackpointbyID(z);
	      int y;
	      y = helpvar;
	      while(f.getTransNextTrackpoint()) {
		f.deleteTrackpointOfTrans(f.getTransID(), f.getTransTrackpointID());
		f.getTransTrackpointbyID(z);
	      } 
	      if(testJoin(helpvar,z)) {
		refreshOtherJoins(helpvar,z);
		testjoin = true;
	      }
	      if(testjoin){
		changeJoinTarget(helpvar,1000,-1);
	      } else {
		f.modifyTrans(helpvar, f.getTransStart(), -1);
	      }
	      redraw(); 
	      f.getTransbyID(y); 
	      f.getTransTrackpointbyID(z);
	      startx = f.getTransTrackpointPosx();
	      starty = f.getTransTrackpointPosy();
	    } else if (i->rtti() == TrackpointItem::RTTI) { 
	      helpvar = (dynamic_cast<TrackpointItem*>(i))->transid;
	      f.getTransbyID(helpvar);
	      transstart = f.getTransStart();
	      int z;
	      z = (dynamic_cast<TrackpointItem*>(i))->id;
	      f.getTransTrackpointbyID(z);
	      int y;
	      y = helpvar;
	      while(f.getTransNextTrackpoint()) {
		f.deleteTrackpointOfTrans(f.getTransID(), f.getTransTrackpointID());
		f.getTransTrackpointbyID(z);
	      } 
	      if(testJoin(helpvar,z)) {
		refreshOtherJoins(helpvar,z);
		testjoin = true;
	      }
	      if(testjoin) {
		changeJoinTarget(helpvar, 1000, -1);
	      } else {
		f.modifyTrans(helpvar, f.getTransStart(), -1);
	      }
	      redraw(); 
	      f.getTransbyID(y); 
	      f.getTransTrackpointbyID(z);
	      startx = f.getTransTrackpointPosx();
	      starty = f.getTransTrackpointPosy();
	    } else {
	      transstart = -1;
	      transend = -1;
	      addTrans();
	      f.modifyTrans(helpvar , (double) e->pos().x() + 5, (double) e->pos().y() + 5, getCurrentLevel());
	      addTrackpoint();
	      newItem->move(e->pos().x() - 2.5,e->pos().y() - 2.5);
	      placeItem(newItem);
	      newItem = 0; 
	      activateItem(0);
	      startx = e->pos().x();
	      starty = e->pos().y();
	    }
	  }
	} else {
	  if(cil.isEmpty()) { 
	    if(foundstate) { 
	      addTrans();	    
	      foundstate = false;
	      int distance = 29;
	      double cx = (e->pos().x() - startx);
	      double cy = (e->pos().y() - starty);
	      double length = sqrt(cx*cx+cy*cy); 
	      if(length != 0) {
		startx = (int)((cx/(length/distance)) + startx);
		starty = (int)((cy/(length/distance)) + starty);
	      }
	    }
	    addTransline((int)startx, (int)starty, (int)e->pos().x(), (int)e->pos().y());
	    addTrackpoint();
	    newItem->move(e->pos().x() - 2.5,e->pos().y() - 2.5);
	    placeItem(newItem);
	    newItem = 0; 
	    activateItem(0);
	    startx = e->pos().x();
	    starty = e->pos().y();
	    f.modifyTrans(helpvar , (double) startx + 5, (double) starty + 5, getCurrentLevel());
	  } else {
	    Q3CanvasItem* i = cil.takeFirst();
	    if(i->rtti() == StateItem::RTTI) {
	      transend = (dynamic_cast<StateItem*>(i))->id;
	      if(foundstate) {  
		if(transend != transstart) { // avoid Transition draw error!!!!
		  addTrans();
		  int distance = 70;
		  double cx = (e->pos().x() - startx);
		  double cy = (e->pos().y() - starty);
		  double length = sqrt(cx*cx+cy*cy); 
		  double txtposx;
		  double txtposy;
		  if(length != 0) {
		    txtposx = ((cx/(length/distance)) + startx);
		    txtposy = ((cy/(length/distance)) + starty);
		  } else {
		    txtposx = startx + 20;
		    txtposy = starty + 20;
		  }
		  if(length > 80) {
		    f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
		  } else {
		    f.modifyTrans(helpvar , (double) startx + 20, (double) starty + 20, getCurrentLevel());
		  }
		  foundstate = false;
		}
	      }
	      if(!foundstate) {
		if(testJoin(helpvar, 1000)) {
		  changeJoinTarget(helpvar, 1000, transend);
		  refreshOtherJoins(helpvar, 1000);
		} else {
		  f.modifyTrans(helpvar, transstart, transend);
		}
	      
		f.getStatebyID(transend);
		double x = f.getStatePosx(); 
		double y = f.getStatePosy();
		addTransline((int)startx, (int)starty, (int)x, (int)y); 
		transbegin = true;
		if(lock == UNLOCKED) {
		  mode = CHOOSE;
		}
		redraw();
	      }
	    } else if(i->rtti() == Joinpoint::RTTI) {
	      if(foundstate) { 
		addTrans();
		{
		  int distancetxt = 70;
		  double cx = (e->pos().x() - startx);
		  double cy = (e->pos().y() - starty);
		  double length = sqrt(cx*cx+cy*cy); 
		  double txtposx;
		  double txtposy;
		  if(length != 0) {
		    txtposx = ((cx/(length/distancetxt)) + startx);
		    txtposy = ((cy/(length/distancetxt)) + starty);
		  } else {
		    txtposx = startx + 20;
		    txtposy = starty + 20;
		  }
		  if(length > 80) {
		    f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
		  } else {
		    f.modifyTrans(helpvar , (double) e->pos().x() + 5, (double) e->pos().y() + 5, getCurrentLevel());
		  }
		}
		foundstate = false;
		int distance = 29;
		double cx = (e->pos().x() - startx);
		double cy = (e->pos().y() - starty);
		double length = sqrt(cx*cx+cy*cy);
		if(length != 0) {
		  startx = (int)((cx/(length/distance)) + startx);
		  starty = (int)((cy/(length/distance)) + starty);
		}
	      }
	      f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
	      f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
	      posx = f.getTransTrackpointPosx();
	      posy = f.getTransTrackpointPosy();
	      addTransline((int)startx, (int)starty, (int)posx, (int)posy); 
	      activateItem(0);
	      f.getTransbyID(helpvar); 
	      transstart = f.getTransStart(); 
	      f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel()); 
	      int z;
	      z = f.getTransTrackpointID();
	      f.modifyTrans(helpvar, z, true);
	      f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
	      f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
	      while (f.getTransNextTrackpoint()) {
		posx = f.getTransTrackpointPosx();
		posy = f.getTransTrackpointPosy(); 
		z = f.getTransTrackpointID();
		int hld;
		hld = f.getTransTrackpointHldirection();
		bool j;
		j =  f.getTransTrackpointJoin();
		bool l;
		l = f.getTransTrackpointLink();
		int lid;
		lid = f.getTransTrackpointLinkID();
		f.getTransbyID(helpvar);
		f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());  
		f.modifyTrans(helpvar, f.getTransTrackpointID(), j);
		f.modifyTrans(helpvar, f.getTransTrackpointID(), l, lid);
		f.transLevel(helpvar, f.getTransTrackpointID(), hld);
		f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
		f.getTransTrackpointbyID(z);
	      } 
	      transend = f.getTransEnd();
	      f.modifyTrans(helpvar, transstart, transend);
	      transbegin = true;
	      if(lock == UNLOCKED) {
		mode = CHOOSE;
	      }
	      f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
	      if((f.getTransStart() < 0)) {
		f.deleteTrans((dynamic_cast<Joinpoint*>(i))->transid);
	      }
	      redraw();
	    } else if(i->rtti() == Linkentrance::RTTI) {
	      if(foundstate) { 
		addTrans();
		int distancetxt = 70;
		double cx = (e->pos().x() - startx);
		double cy = (e->pos().y() - starty);
		double length = sqrt(cx*cx+cy*cy); 
		double txtposx;
		double txtposy;
		if(length != 0) {
		  txtposx = ((cx/(length/distancetxt)) + startx);
		  txtposy = ((cy/(length/distancetxt)) + starty);
		} else {
		  txtposx = startx + 20;
		  txtposy = starty + 20;
		}
		if(length > 80) {
		  f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
		} else {
		  f.modifyTrans(helpvar , (double) e->pos().x() + 5, (double) e->pos().y() + 5, getCurrentLevel());
		}
		foundstate = false;
	      }
	      addTransline((int)startx, (int)starty, (int)e->pos().x(), (int)e->pos().y()); 
	      activateItem(0);
	      f.getTransbyID((dynamic_cast<Linkentrance*>(i))->transid);
	      f.getTransTrackpointbyID((dynamic_cast<Linkentrance*>(i))->id);
	      posx = f.getTransTrackpointPosx();
	      posy = f.getTransTrackpointPosy(); 
	      int w;
	      w = f.getTransTrackpointLinkID();
	      transend = f.getTransEnd();
	      f.modifyTrans(helpvar, transstart, transend);
	      transbegin = true;  
	      f.getTransbyID(helpvar);
	      f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel()); 
	      f.modifyTrans(helpvar, f.getTransTrackpointID(), true, w);
	      if(lock == UNLOCKED) {
		mode = CHOOSE;
	      }
	      f.getTransbyID((dynamic_cast<Linkentrance*>(i))->transid);
	      if(f.getTransStart() < 0) {
		f.deleteTrans((dynamic_cast<Linkentrance*>(i))->transid);
	      }
	      redraw();
	    } else if(i->rtti() == Leveltransfer::RTTI) {
	      if(foundstate) { 
		addTrans();
		int distancetxt = 70;
		double cx = (e->pos().x() - startx);
		double cy = (e->pos().y() - starty);
		double length = sqrt(cx*cx+cy*cy);
		double txtposx; 
		double txtposy;
		if(length != 0) {
		  txtposx = ((cx/(length/distancetxt)) + startx);
		  txtposy = ((cy/(length/distancetxt)) + starty);
		} else {
		  txtposx = startx + 20;
		  txtposy = starty + 20;
		}
		if(length > 80) {
		  f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
		} else {
		  f.modifyTrans(helpvar , (double) e->pos().x() + 5, (double) e->pos().y() + 5, getCurrentLevel());
		}
		foundstate = false;
	      }
	      addTransline((int)startx, (int)starty, (int)(dynamic_cast<Leveltransfer*>(i))->x(), (int)(dynamic_cast<Leveltransfer*>(i))->y());
	      activateItem(0);
	      f.getTransbyID((dynamic_cast<Leveltransfer*>(i))->transid);
	      f.getTransTrackpointbyID((dynamic_cast<Leveltransfer*>(i))->id);
	      posx = f.getTransTrackpointPosx();
	      posy = f.getTransTrackpointPosy(); 
	      transend = f.getTransEnd(); 
	      int w;
	      w = f.getTransTrackpointHldirection();
	      f.getTransbyID(helpvar);
	      f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel()); 
	      f.transLevel(helpvar, f.getTransTrackpointID(), w);
	      f.modifyTrans(helpvar, transstart, transend);
	      transbegin = true;
	      if(lock == UNLOCKED) {
		mode = CHOOSE;
	      } 
	      f.getTransbyID((dynamic_cast<Leveltransfer*>(i))->transid);
	      if((f.getTransStart() < 0) && (f.getTransEnd() < 0)) {
		f.deleteTrans((dynamic_cast<Leveltransfer*>(i))->transid);
	      }
	      redraw();
	    } else {
	      if(foundstate) {
		addTrans();
		foundstate = false;
		int distance = 29;
		double cx = (e->pos().x() - startx);
		double cy = (e->pos().y() - starty);
		double length = sqrt(cx*cx+cy*cy);
		if(length != 0) {
		  startx = (int)((cx/(length/distance)) + startx);
		  starty = (int)((cy/(length/distance)) + starty);
		}
	      }
	      addTransline((int)startx, (int)starty, (int)e->pos().x(), (int)e->pos().y());
	      addTrackpoint();
	      newItem->move(e->pos().x() - 2.5,e->pos().y() - 2.5);
	      placeItem(newItem);
	      newItem = 0;
	      activateItem(0);
	      startx = e->pos().x();
	      starty = e->pos().y();
	      f.modifyTrans(helpvar , (double) startx + 5, (double) starty + 5, getCurrentLevel());
	    }
	  }
	}
      }
      break;
    case HTRANS:
      {
	(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	transbegin = true;
	addHypertrans();
	if(helpvar > -1) {
	  newItem->move(e->pos().x(),e->pos().y());
	  placeItem(newItem);
	  newItem = 0;
	}
	//cout << "Draw Hypertransition" << endl;
	if(lock == UNLOCKED) {
	  mode = CHOOSE;
	}
      }
      break;
    case LINK:
      {
	(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	transbegin = true;
	addLink();
	if(helpvar > -1) {
	  newItem->move(e->pos().x(),e->pos().y());
	  placeItem(newItem);
	  newItem = 0;
	}
	//cout << "Draw Link" << endl;
	if(lock == UNLOCKED) {
	  mode = CHOOSE;
	}
      }
      break;
    case LINKCONNECT: 
      {
	(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	transbegin = true;
	addLinkentrance();
	if(helpvar > -1) {
	  newItem->move(e->pos().x(),e->pos().y());
	  placeItem(newItem);
	  newItem = 0;
	}
	//cout << "Draw Link" << endl;
	if(lock == UNLOCKED) {
	  mode = CHOOSE;
	}
      }
      break;
    case JOIN:
      {
	(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	transbegin = true;
	addJoin();
	newItem->move(e->pos().x(),e->pos().y());
	placeItem(newItem);
	newItem = 0;
	//cout << "Draw Join" << endl;
	if(lock == UNLOCKED) {
	  mode = CHOOSE;
	}
      }
      break;
    case REMOVE: 
      {
	transbegin = true;
	if(cil.empty()) {
	  activateItem(0);
	} else {
	  activateItem(*cil.begin());
	}
	remove();
	//cout << "Remove activated item" << endl;
	if(lock == UNLOCKED) {
	  mode = CHOOSE;
	}
      }
      break;
    case HIERARCHY:
      {
	(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	transbegin = true;
	addHierarchy();
	if(helpvar = -1) {
	  newItem->move(e->pos().x(),e->pos().y());
	  placeItem(newItem);
	  newItem = 0;
	}
	//cout << "Draw Hierarchy" << endl;
	if(lock == UNLOCKED) {
	  mode = CHOOSE;
	}
      }
      break; 
    case TRACKPOINT:
      {
	transbegin = true;
	int id = f.getTransID();
	int trid = f.getTransTrackpointID();
	f.modifyTrans(id, trid, (double)e->pos().x(), (double)e->pos().y());
	if(testjoin) {
	  refreshOtherJoins(id, trid);
	}
	//cout << "Draw Trackpoint" << endl;
	mode = CHOOSE;
	redraw();
	break;
      }
    case Default:
      {
	transbegin = true;
	transinserted = false;
	if(cil.empty()) {
	  activateItem(0);
	} else {
	  drag = true;
	  QApplication::setOverrideCursor(Qt::PointingHandCursor);
	  activateItem(*cil.begin());
	  canvas()->update();
	}
      }
      //cout << "CHOOSE" << endl;
    }
  }
 }
}

void View::contentsMouseMoveEvent(QMouseEvent* e) {
  if(paintmode) {
    mousemoved = true;
    if(activatedItem) {
      if(drag) {
	if(!transinserted && ((activatedItem->rtti()) != (Transline::RTTI))) {
	  activatedItem->move(e->pos().x(),e->pos().y());
	  canvas()->update();
	}
      }
    }
  }
}

void View::contentsMouseReleaseEvent(QMouseEvent* e) {
  if (paintmode) {
    drag = false;
    QApplication::restoreOverrideCursor();
    if(activatedItem) {
      if(mousemoved) {
	if(!transinserted && ((activatedItem->rtti()) != (Transline::RTTI))) {
	  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	  if((e->pos().x() > 0) && (e->pos().y() > 0) && (e->pos().y() < canvas()->height()) && (e->pos().x() < canvas()->width())) {
	    activatedItem->move(e->pos().x(),e->pos().y());
	    switch(activatedItem->rtti()) {
	    case StateItem::RTTI:
	      (dynamic_cast<StateItem*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      break;
	    case Leveltransfer::RTTI:
	      (dynamic_cast<Leveltransfer*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      if(testJoin((dynamic_cast<Leveltransfer*>(activatedItem))->transid, (dynamic_cast<Leveltransfer*>(activatedItem))->id)) {
		refreshOtherJoins((dynamic_cast<Leveltransfer*>(activatedItem))->transid, (dynamic_cast<Leveltransfer*>(activatedItem))->id);
	      }
	      break;
	    case Hypertransition::RTTI:
	      (dynamic_cast<Hypertransition*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      break;
	    case LinkItem::RTTI:
	      (dynamic_cast<LinkItem*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      break;
	    case TrackpointItem::RTTI:
	      (dynamic_cast<TrackpointItem*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      if(testJoin((dynamic_cast<TrackpointItem*>(activatedItem))->transid, (dynamic_cast<TrackpointItem*>(activatedItem))->id)) {
		refreshOtherJoins((dynamic_cast<TrackpointItem*>(activatedItem))->transid, (dynamic_cast<TrackpointItem*>(activatedItem))->id);
	      }
	      break;
	    case Linkentrance::RTTI:
	      (dynamic_cast<Linkentrance*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      if(testJoin((dynamic_cast<Linkentrance*>(activatedItem))->transid, (dynamic_cast<Linkentrance*>(activatedItem))->id)) {
		refreshOtherJoins((dynamic_cast<Linkentrance*>(activatedItem))->transid, (dynamic_cast<Linkentrance*>(activatedItem))->id);
	      }
	      break;
	    case Hierarchyexit::RTTI:
	      (dynamic_cast<Hierarchyexit*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      if(testJoin((dynamic_cast<Hierarchyexit*>(activatedItem))->transid, (dynamic_cast<Hierarchyexit*>(activatedItem))->id)) {
		refreshOtherJoins((dynamic_cast<Hierarchyexit*>(activatedItem))->transid, (dynamic_cast<Hierarchyexit*>(activatedItem))->id);
	      }
	      break;
	    case Joinpoint::RTTI:
	      (dynamic_cast<Joinpoint*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      break;
	    case TranslineText::RTTI:
	      (dynamic_cast<TranslineText*>(activatedItem))->modifyPos(e->pos().x(), e->pos().y());
	      break;
	    }
	  }
	  redraw();
	  canvas()->update();
	}
	mousemoved = false;
      }
    }
  }
}

void View::contentsContextMenuEvent(QContextMenuEvent* e) {
  
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm(); 
  QMenu contextMenu(this);
  mode = CHOOSE;

  if(activatedItem) {
    if(activatedItem->rtti() == Transline::RTTI) {
      if((dynamic_cast<Transline*>(activatedItem))->id > -1) {
	contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
	contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
	contextMenu.addSeparator();
	int helpervar;
	f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
	if(f.getTransStart() != f.getTransEnd()) {
	  helpervar = f.getTransStart();
	  if(helpervar > -1) {
	    f.getStatebyID(helpervar);
	    if(f.getStateLevel() == getCurrentLevel()) {
	      helpervar = f.getTransEnd();
	      if(helpervar > -1) {
		f.getStatebyID(helpervar);
		if(f.getStateLevel() == getCurrentLevel()) {
		  contextMenu.addAction(Straight); 
		}
	      }
	    }
	  }
	}
	contextMenu.addAction(setDefault);
	bool trackpointpossible = true;
	f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
	if(f.getTransFirstTrackpoint()) {
	  do {
	    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	      if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
		trackpointpossible = false;
	      }
	    }
	  } while (f.getTransNextTrackpoint());
	  if(trackpointpossible) {
	    contextMenu.addAction(insertTrackpoint);
	  }
	} else {
	  contextMenu.addAction(insertTrackpoint);
	}
	f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
	if(f.getTransFirstTrackpoint()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	    contextMenu.addAction(change_Start);
	  }
	} else {
	  contextMenu.addAction(change_Start);
	}
	bool targetpossible = true;
	bool linked = false;
	if(f.getTransFirstTrackpoint()) {
	  do {
	    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	      targetpossible = true;
	    } else {
	      targetpossible = false;
	    }
	    if(f.getTransTrackpointLink()) {
	      linked = true;
	    }
	  } while (f.getTransNextTrackpoint());
	  if(targetpossible && !linked) {
	    contextMenu.addAction(change_Target);
	  }
	} else {
	  contextMenu.addAction(change_Target);
	}
	contextMenu.addSeparator();
	contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
      }
    } else if (activatedItem->rtti() == TrackpointItem::RTTI) {
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
      contextMenu.addSeparator();
      contextMenu.addAction(insertTrackpointbefore); 
      contextMenu.addAction(insertTrackpointafter); 
      contextMenu.addAction(connecttoLink);
      f.getTransbyID((dynamic_cast<TrackpointItem*>(activatedItem))->transid);
      int nostart = f.getTransStart();
      if(nostart == -1) {
	contextMenu.addAction(connecttoHierarchy);
      }
      contextMenu.addAction(changetoHierarchy);  
      contextMenu.addAction(createJoin);
      contextMenu.addSeparator();
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
    } else if (activatedItem->rtti() == Joinpoint::RTTI) {   
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back); 
      contextMenu.addSeparator();
      contextMenu.addAction(transformToTrackpoint); 
      contextMenu.addSeparator();
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
    } else if (activatedItem->rtti() == Linkentrance::RTTI) {
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back); 
      contextMenu.addSeparator();
      contextMenu.addAction(transformToTrackpoint);
      contextMenu.addSeparator();
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
    } else if (activatedItem->rtti() == Hierarchyexit::RTTI) {
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back); 
      contextMenu.addSeparator();
      contextMenu.addAction(transformToTrackpoint);
      contextMenu.addSeparator();
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete); 
    } else if (activatedItem->rtti() == TranslineText::RTTI) {
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back); 
      contextMenu.addSeparator();
      contextMenu.addAction(actionresetposition);
    } else {
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
      contextMenu.addSeparator();
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionCu_t);
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Copy);  
      contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
    }
  } else {
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Paste);
    contextMenu.addSeparator();
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_State);
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Transition);
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Hypertrans);
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Link);
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionconnect_to_Link);
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Join);
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_New_Hierarchy);
    contextMenu.addSeparator();
    contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))-> action_Grid);
  }   
  contextMenu.exec(e->globalPos());
}

void View::keyPressEvent(QKeyEvent* ke) {
  if(ke->key() == Qt::Key_Control) {  
    lock = LOCKED; 
    //cout << "LOCKED" << endl;
  }
}

void View::keyReleaseEvent(QKeyEvent* ke) {  
   if(ke->key() == Qt::Key_Control) { 
     //cout << "UNLOCKED" << endl;
     lock = UNLOCKED;
   }
}

void View::setunlocked(){
  lock = UNLOCKED;
}

void View::setlocked() {
  lock = LOCKED;
}

void View::setmode(Workbenchmode w) {
  if(w == TRANS) {
    transbegin = true;
  }
  mode = w;
}

void View::cut() {
  copy();
  remove();
}

void View::copy() { 
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm(); 
  if(activatedItem) {
    posx = 50;
    posy = 50;
    QString s;
    string str = "";
    string str2 = "";
    int id;
    
    switch(activatedItem->rtti()) {    
    case StateItem::RTTI:
      id = dynamic_cast<StateItem*>(activatedItem)->id;
      f.getStatebyID(id);
      s = QString("State %1 %2")
	.arg(QString::fromStdString(f.getStateOutput()))
	.arg(QString::number(f.getStateColor(), 10));
      break;
    case Leveltransfer::RTTI:
      s = QString("Leveltransfer");
      break;
    case Hypertransition::RTTI:
      id = dynamic_cast<Hypertransition*>(activatedItem)->id;
      f.getHypertransbyID(id);
      if(f.getHypertransFirstCondition()) {
	do {
	  str = str + f.getHypertransConditionValue(); 
	  str2 = str2 + "+" + f.getHypertransConditionName();
	} while (f.getHypertransNextCondition());
	str2.assign(str2, 1, str2.length() - 1);
      }
      s = QString("Hypertransition %1 %2")
	.arg(QString::fromStdString(str))
	.arg(QString::fromStdString(str2));
      break;
    case LinkItem::RTTI:
      s = QString("Link");
      break;
    case Linkentrance::RTTI:
      s = QString("Linkentrance");
      break;
    }
    QApplication::clipboard()->setText(s);
  }
}

void View::paste() {
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm(); 
  int id;
  QString s = QApplication::clipboard()->text();
  QTextStream in(&s);
  QString tag;
  in >> tag;
  if ( tag == "State" ) {
    addState();
    newItem->move(posx,posy);
    placeItem(newItem);
    int color;
    QString output;
    in >> output >> color;
    id = dynamic_cast<StateItem*>(newItem)->id;
    f.getStatebyID(id);
    f.modifyState(id, color);
    f.stateOutput(id, output.toStdString());
    newItem = 0;
  } else if ( tag == "Link" ) {
    addLink(); 
    newItem->move(posx,posy);
    placeItem(newItem);
    newItem = 0;
  } else if ( tag == "Leveltransfer" ) {
    addHierarchy();
    newItem->move(posx,posy);
    placeItem(newItem);
    newItem = 0;
  } else if ( tag == "Hypertransition" ) {
    addHypertrans();
    newItem->move(posx,posy);
    placeItem(newItem);
    QString allcon;
    QString conname;
    in >> allcon >> conname;
    string allcons = allcon.toStdString();
    string connames = conname.toStdString();
    if(!allcons.empty()) {
      id = dynamic_cast<Hypertransition*>(newItem)->id;
      f.getHypertransbyID(id);
      string str = "";
      string str2 = "";
      while (allcons.length() > f.getNumberOfInputs()) {
	str.assign(allcons, allcons.length() - f.getNumberOfInputs(), allcons.length() - 1);
	allcons.assign(allcons, 0, allcons.length()  - f.getNumberOfInputs());
	f.addConditionToHypertrans(id);
	f.modifyHypertransConditionValue(id, f.getHypertransConditionID(), str);    
	int len = conname.length();
	if(len > 0) { 
	  do {
	    len--;
	  } while ( (conname.at(len) != '+') && (len != 0));
	  conname.remove(len, conname.length() - 1);
	  str2.assign(connames, len+ 1, connames.length() - len);
	  connames.assign(connames, 0, len);
	}
	
	f.modifyHypertrans(id, f.getHypertransConditionID(), str2);	
      }	
      f.addConditionToHypertrans(id);
      f.modifyHypertransConditionValue(id, f.getHypertransConditionID(), allcons);
      f.modifyHypertrans(id, f.getHypertransConditionID(), connames);
    }
    activateItem(0);
    newItem = 0;
  } else if ( tag == "Link" ) {
    addLinkentrance(); 
    newItem->move(posx,posy);
    placeItem(newItem);
    newItem = 0;
  }
}

void View::remove() {
  if(activatedItem) {   
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Q3CanvasItem* i = activatedItem;	 
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    int id;
    int tid;
    bool go;
    switch(activatedItem->rtti()) {
    case StateItem::RTTI:
      id = (dynamic_cast<StateItem*>(activatedItem))->id;
      f.deleteState(id);
      break;
    case Leveltransfer::RTTI:
      {
      id = (dynamic_cast<Leveltransfer*>(activatedItem))->id;
      tid = (dynamic_cast<Leveltransfer*>(activatedItem))->transid;
      testJoin((dynamic_cast<Leveltransfer*>(activatedItem))->transid, (dynamic_cast<Leveltransfer*>(activatedItem))->id);
      f.getTransbyID(tid);
      f.getTransTrackpointbyID(id);
      int goalState = f.getTransEnd();
      int l = f.getTransStart();
      if(f.getTransNextTrackpoint()) {
	f.addTrans(-1, goalState);
	int newtransid = f.getTransID();
	f.getTransbyID(tid);
	f.getTransTrackpointbyID(id);
	f.getTransNextTrackpoint();
	do {
	  int hl = f.getTransTrackpointLevel();
	  double x = f.getTransTrackpointPosx();
	  double y = f.getTransTrackpointPosy();
	  f.getTransbyID(newtransid);
	  f.addTrackpointToTrans(newtransid, 1000, true, x, y, hl);
	  int trid = f.getTransTrackpointID();
	  f.getTransbyID(tid);
	  f.getTransTrackpointbyID(id);	
	  f.getTransNextTrackpoint();
	  if(f.getTransTrackpointLink()) {
	    f.modifyTrans(newtransid, trid, true, f.getTransTrackpointLinkID());
	  }
	  if(f.getTransTrackpointJoin()) {
	    f.modifyTrans(newtransid, trid, true) ;
	  }
	  if(f.getTransTrackpointLevel() != f.getTransTrackpointHldirection()) {
	    f.transLevel(newtransid, trid, f.getTransTrackpointHldirection());
	  }
	  int deleteid = f.getTransTrackpointID();
	  f.deleteTrackpointOfTrans(tid, deleteid);
	  f.getTransTrackpointbyID(id);
	} while (f.getTransNextTrackpoint());
      }
      f.getTransbyID(tid);
      f.getTransFirstTrackpoint();
      if(f.getTransTrackpointID()==id) {
	f.deleteTrans(tid);
      } else {
	f.deleteTrackpointOfTrans(tid, id);
      }
      if(testjoin) {
	refreshOtherJoins((dynamic_cast<Leveltransfer*>(activatedItem))->transid, (dynamic_cast<Leveltransfer*>(activatedItem))->id);
	testjoin = true;
      }
      if(testjoin) {
	changeJoinTarget(tid, 1000, -1);
      } else {
	f.modifyTrans(tid, l, -1);
      }
      }
      break;
    case Hypertransition::RTTI:
      id = (dynamic_cast<Hypertransition*>(activatedItem))->id;
      f.deleteHypertrans(id);
      break;
    case LinkItem::RTTI:
      id = (dynamic_cast<LinkItem*>(activatedItem))->id;
      f.deleteLink(id);
      break;
    case TrackpointItem::RTTI:
      id = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
      tid = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
      f.getTransbyID(tid);
      go = true;
      if(f.getTransStart() == f.getTransEnd()) {
	if(f.getTransFirstTrackpoint()) {
	  if(!f.getTransNextTrackpoint()) {
	    go = false;
	    f.deleteTrans(tid);
	  }
	}
      }
      if((f.getTransStart() == -1) || (f.getTransEnd() == -1)) {
	if(f.getTransFirstTrackpoint()) {
	  if(!f.getTransNextTrackpoint()) {
	    go = false;
	    f.deleteTrans(tid);
	  }
	}
      }
      if(go) {
	testJoin((dynamic_cast<TrackpointItem*>(activatedItem))->transid, (dynamic_cast<TrackpointItem*>(activatedItem))->id);
	f.deleteTrackpointOfTrans(tid, id);
	if(testjoin) {
	  refreshOtherJoins((dynamic_cast<TrackpointItem*>(activatedItem))->transid, (dynamic_cast<TrackpointItem*>(activatedItem))->id);
	}
      }
      break;
    case Linkentrance::RTTI:
      {
	id = (dynamic_cast<Linkentrance*>(activatedItem))->id;
	tid = (dynamic_cast<Linkentrance*>(activatedItem))->transid;
	testJoin((dynamic_cast<Linkentrance*>(activatedItem))->transid, (dynamic_cast<Linkentrance*>(activatedItem))->id);
	f.getTransbyID(tid);
	int transstart = f.getTransStart();
	f.deleteTrackpointOfTrans(tid, id);
	if(testjoin) {
	  refreshOtherJoins((dynamic_cast<Linkentrance*>(activatedItem))->transid, (dynamic_cast<Linkentrance*>(activatedItem))->id);
	  testjoin = true;
	} 
	if(testjoin) {
	  changeJoinTarget(tid, 1000, -1);
	} else {
	  f.modifyTrans(tid, transstart, -1);
	}
      }
      break;
    case Hierarchyexit::RTTI:
      {
	id = (dynamic_cast<Hierarchyexit*>(activatedItem))->id;
	tid = (dynamic_cast<Hierarchyexit*>(activatedItem))->transid;
	testJoin((dynamic_cast<Hierarchyexit*>(activatedItem))->transid, (dynamic_cast<Hierarchyexit*>(activatedItem))->id);
	f.getTransbyID(tid);
	f.getTransTrackpointbyID(id); 
	int goalS = f.getTransEnd();
	int k = f.getTransStart();
	if(f.getTransNextTrackpoint()) {
	  f.addTrans(-1, goalS);
	  int newtransid = f.getTransID();
	  f.getTransbyID(tid);
	  f.getTransTrackpointbyID(id);
	  f.getTransNextTrackpoint();
	  do {
	    int hl = f.getTransTrackpointLevel();
	    double x = f.getTransTrackpointPosx();
	    double y = f.getTransTrackpointPosy();
	    f.getTransbyID(newtransid);
	    f.addTrackpointToTrans(newtransid, 1000, true, x, y, hl);
	    int trid = f.getTransTrackpointID();
	    f.getTransbyID(tid);
	    f.getTransTrackpointbyID(id);	
	    f.getTransNextTrackpoint();
	    if(f.getTransTrackpointLink()) {
	      f.modifyTrans(newtransid, trid, true, f.getTransTrackpointLinkID());
	    }
	    if(f.getTransTrackpointJoin()) {
	      f.modifyTrans(newtransid, trid, true) ;
	    }
	    if(f.getTransTrackpointLevel() != f.getTransTrackpointHldirection()) {
	      f.transLevel(newtransid, trid, f.getTransTrackpointHldirection());
	    }
	    int deleteid = f.getTransTrackpointID();
	    f.deleteTrackpointOfTrans(tid, deleteid);
	    f.getTransTrackpointbyID(id);
	  } while (f.getTransNextTrackpoint());	
	}
	f.deleteTrackpointOfTrans(tid, id);
	if(testjoin) {
	  refreshOtherJoins((dynamic_cast<Hierarchyexit*>(activatedItem))->transid, (dynamic_cast<Hierarchyexit*>(activatedItem))->id);
	  testjoin = true;
	} 
	if(testjoin) {
	  changeJoinTarget(tid, 1000, -1);
	} else {
	  f.modifyTrans(tid, k, -1);
	}
      }
      break;
    case Joinpoint::RTTI:
      {
	id = (dynamic_cast<Joinpoint*>(activatedItem))->id;
	tid = (dynamic_cast<Joinpoint*>(activatedItem))->transid;
	f.deleteTrackpointOfTrans(tid, id);
      }
      break;
    case Transline::RTTI:
      {
	id = (dynamic_cast<Transline*>(activatedItem))->id;
	if(id > -1) {
	  f.deleteTrans(id);
	}
      }
      break;
    }
    activateItem(0);
    delete i;
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    (dynamic_cast<MainguiWindow*>(wid))->undo();
    (dynamic_cast<MainguiWindow*>(wid))->nomoreredo();
  }
}

void View::movetoback() {
  if(activatedItem){
    --back;
    activatedItem->setZ(back);   
    canvas()->update(); 
  }
}

void View::movetofront() {
  if(activatedItem){
    ++front;
    activatedItem->setZ(front);  
    canvas()->update(); 
  }
}

void View::addState() {
  StateItem* st;
  st = new StateItem(56,56,can);  
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  f.addState();
  int a;
  a = f.getStateID();
  f.stateLevel(a, getCurrentLevel()); 
  //unsigned int col = f.getStateColor();
  st->setBrush(statecolor);
  f.modifyState(a, (unsigned int) statecolor.rgb());
  string s;
  s = f.getStateName(); 
  st->setText(QString::fromStdString(s));
  st->id = a;
  st->modifyPos(posx, posy);
  newItem = st;
}

void View::activateItem(Q3CanvasItem* i) {
  if(i != activatedItem) {
    if(activatedItem){
      if(!paintmode) {
	if(activatedItem->rtti() == StateItem::RTTI) {
	  (dynamic_cast<StateItem*>(activatedItem))->setBrush(QColor(Qt::red)); 
	}
      }
      activatedItem->setActive(false);
    }
    (dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(0,0);
    activatedItem = i;
    if (activatedItem){
      activatedItem->setActive(true); 
      switch(activatedItem->rtti()) {
      case StateItem::RTTI:
	(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(1, (dynamic_cast<StateItem*>(activatedItem))->id); 
	if(!paintmode) {
	  (dynamic_cast<StateItem*>(activatedItem))->setBrush(QColor(Qt::green));
	} 
	break;
      case Transline::RTTI:	
	(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(2, (dynamic_cast<Transline*>(activatedItem))->id);
	break;
      case LinkItem::RTTI:	
	(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(3, (dynamic_cast<LinkItem*>(activatedItem))->id);
	break;
      case Hypertransition::RTTI:	
	(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(4, (dynamic_cast<Hypertransition*>(activatedItem))->id);
	break;
      }
    } 
    canvas()->update(); 
  } 
  canvas()->update(); 
}

void View::placeItem(Q3CanvasItem* i) {
    activateItem(i);
    movetofront();
    i->show();
    canvas()->update();
}

void View::addHypertrans() {
  activateItem(0);
  helpvar = -1;
  NewHyper* nh;
  nh = new NewHyper(this);
  nh->exec();
  if(helpvar > -1) {
    Hypertransition* hy;
    hy = new Hypertransition(20,5,can);
    hy->setBrush(Qt::yellow);
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    f.addHypertrans(true, posx, posy, helpvar, getCurrentLevel());
    int a;
    a = f.getHypertransID();
    hy->id = a;
    Transline* tl;
    tl= new Transline(can);
    a = f.getHypertransGoal();
    f.getStatebyID(a);
    endx = f.getStatePosx();
    endy = f.getStatePosy();
    tl->setBrush(Qt::black);
    int distance = 29;
    double cx = (posx - endx);
    double cy = (posy - endy);
    double length = sqrt(cx*cx+cy*cy);
    if(length != 0) {
      int sx = (int)((cx/(length/distance)) + endx);
      int sy = (int)((cy/(length/distance)) + endy);
      tl->setPoints((int)posx, (int)posy, sx, sy);
      tl->id = -1;
      placeItem(tl);
      drawArrow(posx, posy, endx, endy);
    }
    endx = -1;
    endy = -1;
    newItem = hy;
  }
}

void View::addLink() {
  helpvar = -1;
  NewLink* nl;
  nl = new NewLink(this);
  nl->exec();
  if(helpvar > -1) {
    LinkItem* li;
    li = new LinkItem(30,20,can);
    li->setBrush(Qt::green);
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    f.addLink(posx, posy, helpvar, getCurrentLevel());
    int a;
    a = f.getLinkID();
    li->id = a;
    Transline* tl;
    tl= new Transline(can);
    a = f.getLinkGoal();
    f.getStatebyID(a);
    endx = f.getStatePosx();
    endy = f.getStatePosy();
    tl->setBrush(Qt::black);
    int distance = 29;
    double cx = (posx - endx);
    double cy = (posy - endy);
    double length = sqrt(cx*cx+cy*cy);
    if(length != 0) {
      int sx = (int)((cx/(length/distance)) + endx);
      int sy = (int)((cy/(length/distance)) + endy);
      tl->setPoints((int)posx, (int)posy, sx, sy);
      tl->id = -1;
      placeItem(tl);
      drawArrow(posx, posy, endx, endy);
    }
    endx = -1;
    endy = -1;
    newItem = li;
  }
}


void View::addHierarchy() {
  Leveltransfer* lt;
  lt = new Leveltransfer(30,10,can);
  lt->setBrush(Qt::blue);
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  f.addTrans(-1,-1);
  int id =  f.getTransID();
  lt->transid = id;
  f.addTrackpointToTrans(id, 0, true, posx, posy, getCurrentLevel());
  int trid = f.getTransTrackpointID();
  lt->id = trid;
  helpvar = getCurrentLevel() + 1;
  f.transLevel(id, trid, helpvar);
  newItem = lt;
}


void View::addTransline(int sx, int sy, int ex, int ey) {
  Transline* tl;
  tl = new Transline(can);
  tl->setPoints(sx, sy, ex, ey);
  tl->setBrush(Qt::black);
  tl->id = helpvar;
  placeItem(tl);
}


void View::addTrans() {     
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  f.addTrans(transstart, transend);
  helpvar = f.getTransID();
}

void View::addTrackpoint() {
  TrackpointItem* tp; 
  tp = new TrackpointItem(can);
  tp->setBrush(Qt::black);
  tp->setSize(5,5);
  tp->transid = helpvar;
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  f.addTrackpointToTrans(tp->transid, 1000, true, posx, posy, getCurrentLevel());
  tp->id = f.getTransTrackpointID();
  newItem = tp;
}

void View::addJoin() {
  Joinpoint* j;
  j = new Joinpoint(can);
  j->setBrush(Qt::red);
  j->setSize(10,10);
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  f.addTrans(-1,-1);
  int id =  f.getTransID();
  j->transid = id;
  f.addTrackpointToTrans(id, 0, true, posx, posy, getCurrentLevel());
  int trid = f.getTransTrackpointID();
  j->id = trid;
  f.modifyTrans(id, trid, true);
  newItem = j;
}

void View::addLinkentrance() {
  Linkconnector* lc;
  lc = new Linkconnector(this);
  lc ->exec();
  if(helpvar > -1) {
    Linkentrance* le;
    le = new Linkentrance(can);
    le->setBrush(Qt::green);
    le->setSize(10,5);
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    f.addTrans(-1,-1);
    int id =  f.getTransID();
    le->transid = id;
    f.addTrackpointToTrans(id, 0, true, posx, posy, getCurrentLevel());
    int trid = f.getTransTrackpointID();
    le->id = trid;
    f.modifyTrans(id, trid, true, helpvar); 
    f.getLinkbyID(helpvar);
    int target = f.getLinkGoal();
    int start = f.getTransStart();
    f.modifyTrans(id, start, target);
    newItem = le; 
  }
}

void View::grid() {
  if(usegrid) {
    usegrid = false;
    canvas()->setBackgroundPixmap(QPixmap());
  } else {  
    usegrid = true;
    canvas()->setBackgroundPixmap(*bgGrid);
  }
}

void View::zoomIn() {
  QMatrix wm=worldMatrix();
  wm.scale(1.1,1.1);
  setWorldMatrix(wm);
}

void View::zoomOut() {
  QMatrix wm=worldMatrix();
  wm.scale(1/1.1,1/1.1);  
  setWorldMatrix(wm);
}

void View::createActionsforContextMenu() {
  Straight =  new QAction((dynamic_cast<MainguiWindow*>(wid))); 
  Straight->setText(QApplication::translate("View", "Straight", 0, QApplication::UnicodeUTF8));
  Straight->setObjectName(QString::fromUtf8("Straight"));
  setDefault =  new QAction((dynamic_cast<MainguiWindow*>(wid))); 
  setDefault->setObjectName(QString::fromUtf8("setDefault"));
  setDefault->setText(QApplication::translate("View", "Default", 0, QApplication::UnicodeUTF8));
  change_Start =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  change_Start->setObjectName(QString::fromUtf8("change_Start"));
  change_Start->setText(QApplication::translate("View", "change Start", 0, QApplication::UnicodeUTF8));
  change_Target =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  change_Target->setObjectName(QString::fromUtf8("change_Target"));
  change_Target->setText(QApplication::translate("View", "change Target", 0, QApplication::UnicodeUTF8));
  insertTrackpointbefore  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  insertTrackpointbefore->setObjectName(QString::fromUtf8("insertTrackpointbefore"));
  insertTrackpointbefore->setText(QApplication::translate("View", "New Trackpoint before", 0, QApplication::UnicodeUTF8));
  insertTrackpointafter  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  insertTrackpointafter->setObjectName(QString::fromUtf8("insertTrackpointafter"));
  insertTrackpointafter->setText(QApplication::translate("View", "New Trackpoint after", 0, QApplication::UnicodeUTF8)); 
  insertTrackpoint  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  insertTrackpoint->setObjectName(QString::fromUtf8("insertTrackpoint"));
  insertTrackpoint->setText(QApplication::translate("View", "New Trackpoint", 0, QApplication::UnicodeUTF8));
  connecttoLink =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  connecttoLink->setObjectName(QString::fromUtf8("connecttoLink"));
  connecttoLink->setText(QApplication::translate("View", "connect to Link", 0, QApplication::UnicodeUTF8));
  changetoHierarchy  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  changetoHierarchy->setObjectName(QString::fromUtf8("changetoHierarchy"));
  changetoHierarchy->setText(QApplication::translate("View", "change to Hierarchy", 0, QApplication::UnicodeUTF8));
  createJoin  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  createJoin->setObjectName(QString::fromUtf8("createJoin"));
  createJoin->setText(QApplication::translate("View", "change to Join", 0, QApplication::UnicodeUTF8));
  transformToTrackpoint  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  transformToTrackpoint->setObjectName(QString::fromUtf8("transformToTrackpoint"));
  transformToTrackpoint->setText(QApplication::translate("View", "Transform to Trackpoint", 0, QApplication::UnicodeUTF8)); 
  connecttoHierarchy =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  connecttoHierarchy->setObjectName(QString::fromUtf8("connecttoHierarchy"));
  connecttoHierarchy->setText(QApplication::translate("View", "connect to Hierarchy", 0, QApplication::UnicodeUTF8));
  actionresetposition =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
  actionresetposition->setObjectName(QString::fromUtf8("resetposition"));
  actionresetposition->setText(QApplication::translate("View", "Reset Position", 0, QApplication::UnicodeUTF8));
}

void View::straighten() { 
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  if(!((dynamic_cast<Transline*>(activatedItem))->id < 0)) {
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
    if((f.getTransStart() < 0) || (f.getTransEnd() < 0)) {
      f.deleteTrans((dynamic_cast<Transline*>(activatedItem))->id);
    } else {
      while(f.getTransFirstTrackpoint()) {
	int i = f.getTransTrackpointID();
	f.deleteTrackpointOfTrans((dynamic_cast<Transline*>(activatedItem))->id, i);
      }
      redraw();	
      canvas()->update();
    }
  }
}

void View::setdefault() {
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  if(!((dynamic_cast<Transline*>(activatedItem))->id < 0)) {
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
    if(f.getTransDefault()) {
      f.modifyTrans((dynamic_cast<Transline*>(activatedItem))->id, false);
    } else {  
      f.modifyTrans((dynamic_cast<Transline*>(activatedItem))->id, true);
    } 
    newItem = activatedItem;
    activateItem(0);
    activateItem(newItem);
    newItem = 0;
  }
}

void View::changestart() {
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  if(!((dynamic_cast<Transline*>(activatedItem))->id < 0)) {
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
    int end;
    end = f.getTransEnd();
    ChangeStart* cs;
    cs = new ChangeStart(this);
    cs->exec();
    if(helpvar > -1) {
      f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
      if(f.getTransFirstTrackpoint()) {
	f.modifyTrans((dynamic_cast<Transline*>(activatedItem))->id, helpvar, end);
      } else {
	if(end != helpvar) {
	  f.modifyTrans((dynamic_cast<Transline*>(activatedItem))->id, helpvar, end);
	}
      }
    } 
    redraw();
  }
}

void View::changetarget() {
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  if(!((dynamic_cast<Transline*>(activatedItem))->id < 0)) {
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    int id = (dynamic_cast<Transline*>(activatedItem))->id;
    f.getTransbyID(id);
    int start;
    start = f.getTransStart();
    ChangeEnd* ce;
    ce = new ChangeEnd(this);
    ce->exec();
    if(helpvar > -1) {
      if(testJoin(id, 1000)) {
	changeJoinTarget(id, 1000, helpvar);
	refreshOtherJoins(id, 1000);
      } else {
	f.getTransbyID(id);
	if(f.getTransFirstTrackpoint()) {
	  f.modifyTrans(id, start, helpvar);
	} else {
	  if(start != helpvar) {
	    f.modifyTrans(id, start, helpvar);
	  }
	}	
      }
    }
    redraw();
  }
}

void View::inserttrackpoint() {  
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  bool ok = true;
  int id = (dynamic_cast<Transline*>(activatedItem))->id;
  testJoin(id, 1000);
  f.getTransbyID(id);
  if(f.getTransFirstTrackpoint()) {
    if((f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) && (f.getTransTrackpointLevel() == getCurrentLevel())) {
      ok = false;
    }
    while(f.getTransNextTrackpoint()){
      if((f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) && (f.getTransTrackpointLevel() == getCurrentLevel())) {
	ok = false;
      }
    }
  }
  if(ok) {
    mode =  TRACKPOINT; 
    f.addTrackpointToTrans(id, 1000, true, 10, 10, getCurrentLevel());
  }
}

void View::inserttrackpointbefore() {  
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  mode =  TRACKPOINT; 
  int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
  int posid = (dynamic_cast<TrackpointItem*>(activatedItem))->id; 
  testJoin(id, posid);
  f.getTransbyID(id);
  f.getTransTrackpointbyID(posid);
  f.addTrackpointToTrans(id, posid, true, 10, 10, getCurrentLevel());
}

void View::inserttrackpointafter() {  
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  mode =  TRACKPOINT; 
  int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
  int posid = (dynamic_cast<TrackpointItem*>(activatedItem))->id; 
  testJoin(id, posid);
  f.getTransbyID(id);
  f.getTransTrackpointbyID(posid);
  f.addTrackpointToTrans(id, posid, false, 10, 10, getCurrentLevel());
}

void View::connecttolink() { 
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
  int trid = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
  f.getTransbyID(id);
  f.getTransTrackpointbyID(trid);
  Linkconnector* lc;
  lc = new Linkconnector(this);
  lc ->exec();
  if(helpvar > -1) { 
    testJoin(id, trid);
    f.modifyTrans(id, trid, true, helpvar);
    f.getLinkbyID(helpvar);
    int target = f.getLinkGoal();
    int start = f.getTransStart();
    if(testjoin) {
      refreshOtherJoins(id, trid);
      testjoin = true;
    }
    if(testjoin) {
      changeJoinTarget(id, trid, target);
    } else {
      f.modifyTrans(id, start, target);
    } 
    redraw();
  }
}

void View::changetohierarchy() { 
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
  int trid = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
  testJoin(id, trid);
  f.getTransbyID(id);
  f.getTransTrackpointbyID(trid);
  f.transLevel(id, trid, getCurrentLevel() + 1);
  while(f.getTransNextTrackpoint()) {
    f.deleteTrackpointOfTrans(id,f.getTransTrackpointID());
    f.getTransTrackpointbyID(trid);
  }
  f.getTransTrackpointbyID(trid);
  if(testjoin) {
    refreshOtherJoins(id, trid);
  }  
  redraw();
}

void View::createjoin() {
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
  int trid = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
  testJoin(id, trid);
  f.getTransbyID(id);
  f.getTransTrackpointbyID(trid);
  f.modifyTrans((dynamic_cast<TrackpointItem*>(activatedItem))->transid, (dynamic_cast<TrackpointItem*>(activatedItem))->id, true);
  Q3CanvasItem* i = activatedItem;
  activateItem(0);
    delete i;
    drawJoin(); 
    if(testjoin) {
      refreshOtherJoins(id, trid);
    }
}

void View::transformtotrackpoint() {
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  if (activatedItem->rtti() == Joinpoint::RTTI) {
    int id = (dynamic_cast<Joinpoint*>(activatedItem))->transid;
    int trid = (dynamic_cast<Joinpoint*>(activatedItem))->id; 
    testJoin(id, trid);
    activateItem(0);
    f.modifyTrans(id, trid, false); 
    if(testjoin) {
      refreshOtherJoins(id, trid);
    }
  } else if (activatedItem->rtti() == Linkentrance::RTTI) {
    int id = (dynamic_cast<Linkentrance*>(activatedItem))->transid;
    int trid = (dynamic_cast<Linkentrance*>(activatedItem))->id;  
    testJoin(id, trid);
    activateItem(0);
    f.modifyTrans(id, trid, false, 0); 
    f.getTransbyID(id);
    int k = f.getTransStart();
    int l = -1; 
    if(testjoin) {
      refreshOtherJoins(id, trid);
      testjoin = true;
    }
    if(testjoin) {
      changeJoinTarget(id, trid, l);
    } else {
      f.modifyTrans(id, k, l);
    }
  } else if (activatedItem->rtti() == Hierarchyexit::RTTI) {
    int id = (dynamic_cast<Hierarchyexit*>(activatedItem))->id;
    int tid = (dynamic_cast<Hierarchyexit*>(activatedItem))->transid; 
    testJoin(tid, id);
    activateItem(0);
    f.getTransbyID(tid);
    f.getTransTrackpointbyID(id);
    int goalState = f.getTransEnd();
    int l = f.getTransStart(); 
    f.addTrans(-1, goalState);
    int newtransid = f.getTransID();    
    f.getTransbyID(tid);
    f.getTransTrackpointbyID(id);
    while (f.getTransNextTrackpoint()) {
	int hl = f.getTransTrackpointLevel();
	double x = f.getTransTrackpointPosx();
	double y = f.getTransTrackpointPosy();
	f.getTransbyID(newtransid);
	f.addTrackpointToTrans(newtransid, 1000, true, x, y, hl);
	int trid = f.getTransTrackpointID();
	f.getTransbyID(tid);
	f.getTransTrackpointbyID(id);	
	f.getTransNextTrackpoint();
	if(f.getTransTrackpointLink()) {
	  f.modifyTrans(newtransid, trid, true, f.getTransTrackpointLinkID());
	}
	if(f.getTransTrackpointJoin()) {
	  f.modifyTrans(newtransid, trid, true) ;
	}
	if(f.getTransTrackpointLevel() != f.getTransTrackpointHldirection()) {
	  f.transLevel(newtransid, trid, f.getTransTrackpointHldirection());
	}
	int deleteid = f.getTransTrackpointID();
	f.deleteTrackpointOfTrans(tid, deleteid);
	f.getTransTrackpointbyID(id);
    }
    f.getTransbyID(tid);
    f.getTransTrackpointbyID(id);
    int hl = f.getTransTrackpointLevel();
    double x = f.getTransTrackpointPosx();
    double y = f.getTransTrackpointPosy();
    f.getTransbyID(newtransid);
    f.addTrackpointToTrans(newtransid, -10, true, x, y, hl); 
    if(testJoin(tid, id)) {
      changeJoinTarget(tid, id, -1);
      f.deleteTrackpointOfTrans(tid, id); 
      testjoin = true;
    } else {
      int curid = f.getTransStart();
      f.modifyTrans(tid, curid, -1);
      f.deleteTrackpointOfTrans(tid, id); 
    }
    if(testjoin) {
      refreshOtherJoins(tid, 1000);
    }
  }
  redraw();
}

void View::connecttohierarchy() {
  (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
  Hierarchyconnector* hc;
  hc = new Hierarchyconnector(this);
  hc ->exec();
  if(helpvar > -1) { 
    Core_I& c = CoreFactory::getCore_I(); 
    Fsm_I& f = c.getCurrentFsm();
    int tid = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
    int id = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
    int trid;
    f.getTransbyID(tid);
    f.getTransTrackpointbyID(id);
    int hl = f.getTransTrackpointLevel();
    double x = f.getTransTrackpointPosx();
    double y = f.getTransTrackpointPosy();
    f.addTrackpointToTrans(helpvar, 1000, true, x, y, hl);
    f.getTransbyID(tid);
    f.getTransTrackpointbyID(id);
    while(f.getTransNextTrackpoint()) {
      hl = f.getTransTrackpointLevel();
      x = f.getTransTrackpointPosx();
      y = f.getTransTrackpointPosy();
      f.addTrackpointToTrans(helpvar, 1000, true, x, y, hl);
      trid = f.getTransTrackpointID(); 
      f.getTransbyID(tid);
      f.getTransTrackpointbyID(id);	
      f.getTransNextTrackpoint();
      if(f.getTransTrackpointLink()) {
	f.modifyTrans(helpvar, trid, true, f.getTransTrackpointLinkID());
      }
      if(f.getTransTrackpointJoin()) {
	f.modifyTrans(helpvar, trid, true) ;
      }
      if(f.getTransTrackpointLevel() != f.getTransTrackpointHldirection()) {
	f.transLevel(helpvar, trid, f.getTransTrackpointHldirection());
      }
      int deleteid = f.getTransTrackpointID();
      f.deleteTrackpointOfTrans(tid, deleteid);
      f.getTransTrackpointbyID(id);
    } 
    f.getTransbyID(tid);
    int e = f.getTransEnd();
    f.getTransbyID(helpvar);
    int s = f.getTransStart();
    f.deleteTrans(tid);
    if(testJoin(helpvar, 1000)) {
      refreshOtherJoins(helpvar, trid);
      testjoin = true;
    }
    if(testjoin) {
      changeJoinTarget(helpvar, 1000, e);
    } else {
      f.modifyTrans(helpvar, s, e);
    }
    redraw();
  }
}

void View::connectContextMenuActions() {
  connect(Straight,SIGNAL(activated()),this,SLOT(straighten()));
  connect(setDefault,SIGNAL(activated()),this,SLOT(setdefault()));
  connect(change_Start,SIGNAL(activated()),this,SLOT(changestart()));
  connect(change_Target,SIGNAL(activated()),this,SLOT(changetarget()));
  connect(insertTrackpointbefore,SIGNAL(activated()),this,SLOT(inserttrackpointbefore()));
  connect(insertTrackpointafter,SIGNAL(activated()),this,SLOT(inserttrackpointafter()));
  connect(connecttoLink,SIGNAL(activated()),this,SLOT(connecttolink()));
  connect(changetoHierarchy,SIGNAL(activated()),this,SLOT(changetohierarchy()));
  connect(createJoin,SIGNAL(activated()),this,SLOT(createjoin()));
  connect(transformToTrackpoint,SIGNAL(activated()),this,SLOT(transformtotrackpoint()));
  connect(insertTrackpoint,SIGNAL(activated()),this,SLOT(inserttrackpoint()));
  connect(connecttoHierarchy,SIGNAL(activated()),this,SLOT(connecttohierarchy())); 
  connect(actionresetposition,SIGNAL(activated()),this,SLOT(resetposition()));
}

int View::getCurrentLevel() {
  return (dynamic_cast<MainguiWindow*>(wid))->getcurrenthierarchylevel();
}

void View::sethelpvar(int i) {
  helpvar = i;
}

void View::setPaintmode(bool b) {
  paintmode = b;
  if(!b) {
    activateItem(0); 
  }
}

void View::redraw() { 
  activateItem(0);
  Q3CanvasItemList cil = canvas()->allItems();
  while (!cil.isEmpty()) {
    Q3CanvasItem* i = cil.takeFirst();
    delete i;
  }
  back = 0;
  front = 0;
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  if(f.getFirstTrans()) {
      drawTrans();
      drawTransText();
    while(f.getNextTrans()) { 
	drawTrans();
	drawTransText();
    }
  }
  if(f.getFirstTrans()) { 
      drawTransPoints();
    while(f.getNextTrans()) {
	drawTransPoints();
    }
  }
  if(f.getFirstLink()) {
    if(f.getLinkLevel() == getCurrentLevel()) { 
      drawLink();
    }
    while(f.getNextLink()) {
      if (f.getLinkLevel() == getCurrentLevel()) {
	drawLink();
      }
    }
  }
  if(f.getFirstHypertrans()) {
    if(f.getHypertransLevel() == getCurrentLevel()) {
      drawHypertrans();
    }
    while(f.getNextHypertrans()) {
      if(f.getHypertransLevel() == getCurrentLevel()) {
	drawHypertrans();
      }
    }
  }
  if(f.getFirstState()) {
    if(f.getStateLevel() == getCurrentLevel()) {
      drawState();
    }
    while(f.getNextState()) {
      if(f.getStateLevel() == getCurrentLevel()) {
	drawState();
      }
    }
  }
  activateItem(0);
  canvas()->update();
}

void View::drawState() {
  StateItem* st;
  st = new StateItem(56,56,can);
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  int a;
  a = f.getStateID();
  string s;
  s = f.getStateName(); 
  st->setText(QString::fromStdString(s));
  st->id = a;
  posx = f.getStatePosx();
  posy = f.getStatePosy();
  unsigned int color = f.getStateColor();
  st->setBrush(QColor(color));
  st->move(posx,posy);
  placeItem(st);
}

void View::drawHypertrans() {
  Hypertransition* hy;
  hy = new Hypertransition(20,5,can);
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  int a;
  a = f.getHypertransID();
  hy->id = a;
  posx = f.getHypertransPosx();
  posy = f.getHypertransPosy();
  hy->setBrush(Qt::yellow);
  hy->move(posx,posy);
  Transline* tl;
  tl= new Transline(can);
  a = f.getHypertransGoal();
  f.getStatebyID(a);
  endx = f.getStatePosx();
  endy = f.getStatePosy();
  tl->setBrush(Qt::black);
  int distance = 29;
  double cx = (posx - endx);
  double cy = (posy - endy);
  double length = sqrt(cx*cx+cy*cy);
  if(length != 0) {
    int sx = (int)((cx/(length/distance)) + endx);
    int sy = (int)((cy/(length/distance)) + endy);
    tl->setPoints((int)posx, (int)posy, sx, sy);
    tl->id = -1;
    placeItem(tl);
    drawArrow(posx, posy, endx, endy);
  } 
  placeItem(hy);
  endx = -1;
  endy = -1;
}

void View::drawLink() {
  LinkItem* li;
  li = new LinkItem(30,20,can);
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  int a;
  a = f.getLinkID();
  li->id = a;
  posx = f.getLinkPosx();
  posy = f.getLinkPosy();
  li->setBrush(Qt::green);
  li->move(posx,posy);
  Transline* tl;
  tl= new Transline(can);
  a = f.getLinkGoal();
  f.getStatebyID(a);
  endx = f.getStatePosx();
  endy = f.getStatePosy();
  tl->setBrush(Qt::black);
  int distance = 29;
  double cx = (posx - endx);
  double cy = (posy - endy);
  double length = sqrt(cx*cx+cy*cy);
  if(length != 0) {
    int sx = (int)((cx/(length/distance)) + endx);
    int sy = (int)((cy/(length/distance)) + endy);
    tl->setPoints((int)posx, (int)posy, sx, sy);
    tl->id = -1;
    placeItem(tl);
    drawArrow(posx, posy, endx, endy);
  }
  placeItem(li);
  endx = -1;
  endy = -1;
}

void View::drawHierarchy() { 
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  Leveltransfer* lt;
  lt = new Leveltransfer(30,10,can);
  lt->setBrush(Qt::blue);
  int id =  f.getTransID();
  lt->transid = id;
  int trid = f.getTransTrackpointID();
  lt->id = trid;
  posx = f.getTransTrackpointPosx();
  posy = f.getTransTrackpointPosy();
  lt->move(posx, posy);
  placeItem(lt);
}

void View::drawTrans() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  bool connecttoend = true;
  int a;
  int x;
  int y;
  int transid;
  int trackid;
  double posxx;
  double posyy;
  bool draw = false;
  Q3CanvasItemList cil;
  a = f.getTransID();
  if((f.getTransStart() < 0) && (f.getTransEnd() < 0)) {  
    if(f.getTransFirstTrackpoint()) {
      if(f.getTransTrackpointJoin()) {
      } else if(f.getTransTrackpointLink()) {
      } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
      }
      if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	draw = true;
      } else {
	draw = false;
      }
      posxx = f.getTransTrackpointPosx();
      posyy = f.getTransTrackpointPosy();
      while(f.getTransNextTrackpoint()) {
	if(f.getTransTrackpointLevel() != getCurrentLevel()) {
	  draw = false;
	}
	posx = f.getTransTrackpointPosx();
	posy = f.getTransTrackpointPosy(); 
	if(draw) {
	  Transline* tl;
	  tl = new Transline(can);
	  tl->setPoints((int)posxx, (int)posyy, (int)posx, (int)posy);
	  tl->setBrush(Qt::black);
	  tl->id = a;
	  placeItem(tl);
	}
	if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	  draw = true;
	}
	posxx = posx;
	posyy = posy;
	if(f.getTransTrackpointJoin()) {
	  transid = f.getTransID();
	  trackid = f.getTransTrackpointID();
	  int goal = f.getTransEnd();
	  x = (int) f.getTransTrackpointPosx();
	  y = (int) f.getTransTrackpointPosy();
	  cil = canvas()->collisions(QPoint(x, y)); 
	  while (!cil.isEmpty()) {
	    Q3CanvasItem* i = cil.takeFirst();
	    if(i->rtti() == Joinpoint::RTTI) {
	      f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
	      if(f.getTransEnd() == goal) {
		f.getTransbyID(transid);
		f.getTransTrackpointbyID(trackid);
		while(f.getTransNextTrackpoint()) {}
	      } else {
		f.getTransbyID(transid);
		f.getTransTrackpointbyID(trackid);
	      }
	    }
	  }        
	} else if(f.getTransTrackpointLink()) {
	  while(f.getTransNextTrackpoint()) {}
	} else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	    while(f.getTransNextTrackpoint()) {}
	  }
	}	
      }
    } else {
      f.deleteTrans(a);
    }
  } else {
    if(f.getTransStart() < 0) { 
      if(f.getTransFirstTrackpoint()) { 
	if(f.getTransTrackpointJoin()) {
	} else if(f.getTransTrackpointLink()) {
	  while(f.getTransNextTrackpoint()) {}
	  connecttoend = false;
	} else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	    while(f.getTransNextTrackpoint()) {}
	    connecttoend = false;
	  }
	}
	if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	  draw = true;
	} else {
	  draw = false;
	}
	posxx = f.getTransTrackpointPosx();
	posyy = f.getTransTrackpointPosy(); 
	while(f.getTransNextTrackpoint()) {
	  if(!(f.getTransTrackpointLevel() == getCurrentLevel())) {
	    draw = false;
	  }
	  posx = f.getTransTrackpointPosx();
	  posy = f.getTransTrackpointPosy(); 
	  if(draw) {
	    Transline* tl;
	    tl = new Transline(can);
	    tl->setPoints((int)posxx, (int)posyy, (int)posx, (int)posy);
	    tl->setBrush(Qt::black);
	    tl->id = a;
	    placeItem(tl);
	  }
	  if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	    draw = true;
	  }
	  posxx = posx;
	  posyy = posy;
	  if(f.getTransTrackpointJoin()) {
	    transid = f.getTransID();
	    trackid = f.getTransTrackpointID();
	    int goal = f.getTransEnd();
	    x = (int) f.getTransTrackpointPosx();
	    y = (int) f.getTransTrackpointPosy();
	    cil = canvas()->collisions(QPoint(x, y));         
	    while (!cil.isEmpty()) {
	      Q3CanvasItem* i = cil.takeFirst();
	      if(i->rtti() == Joinpoint::RTTI) {
		f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
		if(f.getTransEnd() == goal) {
		  f.getTransbyID(transid);
		  f.getTransTrackpointbyID(trackid);
		  while(f.getTransNextTrackpoint()) {}
		  connecttoend = false;
		} else {
		  f.getTransbyID(transid);
		  f.getTransTrackpointbyID(trackid);
		}
	      }
	    }
	  } else if(f.getTransTrackpointLink()) {
	    while(f.getTransNextTrackpoint()) {}
	    connecttoend = false;
	  } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	      while(f.getTransNextTrackpoint()) {}
	      connecttoend = false;
	    }
	  }	   
	}
	if(!(f.getTransEnd() < 0)  && connecttoend) {
	  f.getStatebyID(f.getTransEnd());
	  posx = f.getStatePosx();
	  posy = f.getStatePosy();
	  if(!(f.getStateLevel() == getCurrentLevel())) {
	    draw = false;
	  }
	  if(draw){
	    Transline* tl;
	    tl = new Transline(can);
	    int distance = 29;
	    double cx = (posxx - posx);
	    double cy = (posyy - posy);
	    double length = sqrt(cx*cx+cy*cy);
	    if(length != 0) {
	      int sx = (int)((cx/(length/distance)) + posx);
	      int sy = (int)((cy/(length/distance)) + posy);
	      tl->setPoints((int)posxx, (int)posyy, sx, sy);
	      tl->setBrush(Qt::black);
	      tl->id = a;
	      placeItem(tl);  
	      drawArrow(posxx, posyy, posx, posy);
	    }
	  }
	}	
      } else {
	f.deleteTrans(a);
      }
    } else {
      f.getStatebyID(f.getTransStart());
      posxx = f.getStatePosx();
      posyy = f.getStatePosy();
      if(f.getStateLevel() == getCurrentLevel()) {
	draw = true;
      } else {
	draw = false;
      }
      if(f.getTransFirstTrackpoint()) {	
	if(f.getTransTrackpointLevel() != getCurrentLevel()) {
	  draw = false;
	}
	if(f.getTransTrackpointJoin()) {
	  transid = f.getTransID();
	  trackid = f.getTransTrackpointID();
	  int goal = f.getTransEnd();
	  posx = f.getTransTrackpointPosx();
	  posy = f.getTransTrackpointPosy();
	  x = (int) posx;
	  y = (int) posy;
	  cil = canvas()->collisions(QPoint(x, y));         
	  while (!cil.isEmpty()) {
	    Q3CanvasItem* i = cil.takeFirst();
	    if(i->rtti() == Joinpoint::RTTI) {
	      f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
	      if(f.getTransEnd() == goal) {
		f.getTransbyID(transid);
		f.getTransTrackpointbyID(trackid);
		while(f.getTransNextTrackpoint()) {}
		connecttoend = false;
	      } else {
		f.getTransbyID(transid);
		f.getTransTrackpointbyID(trackid);
	      }
	    }
	  }
	} else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	  posx = f.getTransTrackpointPosx();
	  posy = f.getTransTrackpointPosy();
	  if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	    while(f.getTransNextTrackpoint()) {}
	    connecttoend = false; 
	  }
	} else if(f.getTransTrackpointLink()) { 	
	  posx = f.getTransTrackpointPosx();
	  posy = f.getTransTrackpointPosy();
	  while(f.getTransNextTrackpoint()) {}
	  connecttoend = false;
	} else {
	  posx = f.getTransTrackpointPosx();
	  posy = f.getTransTrackpointPosy();
	} 
	if(draw) {
	  Transline* tl;
	  tl = new Transline(can);
	  int distance = 29;
	  double cx = (posx - posxx);
	  double cy = (posy - posyy);
	  double length = sqrt(cx*cx+cy*cy);
	  if(length != 0) {
	    posxx = (int)((cx/(length/distance)) + posxx);
	    posyy = (int)((cy/(length/distance)) + posyy);
	    tl->setPoints((int)posxx, (int)posyy, (int)posx, (int)posy);
	    tl->setBrush(Qt::black);
	    tl->id = a;
	    placeItem(tl);
	  }
	}
	if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	  draw = true;
	}
	posxx = posx;
	posyy = posy;
	while(f.getTransNextTrackpoint()) {
	  posx = f.getTransTrackpointPosx();
	  posy = f.getTransTrackpointPosy(); 
	  if(f.getTransTrackpointLevel() != getCurrentLevel()) {
	    draw = false;
	  }
	  if(draw) {
	    Transline* tl;
	    tl = new Transline(can);
	    tl->setPoints((int)posxx, (int)posyy, (int)posx, (int)posy);
	    tl->setBrush(Qt::black);
	    tl->id = a;
	    placeItem(tl);
	  } 
	  if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	    draw = true;
	  }
	  posxx = posx;
	  posyy = posy;
	  if(f.getTransTrackpointJoin()) {
	    transid = f.getTransID();
	    trackid = f.getTransTrackpointID();
	    int goal = f.getTransEnd();
	    x = (int) f.getTransTrackpointPosx();
	    y = (int) f.getTransTrackpointPosy();
	    cil = canvas()->collisions(QPoint(x, y));         
	    while (!cil.isEmpty()) {
	      Q3CanvasItem* i = cil.takeFirst();
	      if(i->rtti() == Joinpoint::RTTI) {
		f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
		if(f.getTransEnd() == goal) {
		  f.getTransbyID(transid);


		  f.getTransTrackpointbyID(trackid);
		  while(f.getTransNextTrackpoint()) {}
		  connecttoend = false;
		} else {
		  f.getTransbyID(transid);
		  f.getTransTrackpointbyID(trackid);
		}
	      }
	    }
	  } else if(f.getTransTrackpointLink()) {
	    while(f.getTransNextTrackpoint()) {}
	    connecttoend = false;
	  } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
	      while(f.getTransNextTrackpoint()) {}
	      connecttoend = false;
	    }
	  }
	}
	if(!(f.getTransEnd() < 0) && connecttoend) {
	  f.getStatebyID(f.getTransEnd());
	  posx = f.getStatePosx();
	  posy = f.getStatePosy();
	  if(f.getStateLevel() != getCurrentLevel()) {
	    draw = false;
	  }
	  if(draw) {
	    Transline* tl;
	    tl = new Transline(can);
	    int distance = 29;
	    double cx = (posxx - posx);
	    double cy = (posyy - posy);
	    double length = sqrt(cx*cx+cy*cy);
	    if(length != 0) {
	      int sx = (int)((cx/(length/distance)) + posx);
	      int sy = (int)((cy/(length/distance)) + posy);
	      tl->setPoints((int)posxx, (int)posyy, sx, sy);
	      tl->setBrush(Qt::black);
	      tl->id = a;
	      placeItem(tl);
	      drawArrow(posxx, posyy, posx, posy);
	    }
	  }
	}
      } else {
	if(f.getTransEnd() < 0) {
	  f.deleteTrans(a);
	} else {
	  f.getStatebyID(f.getTransEnd());
	  posx = f.getStatePosx();
	  posy = f.getStatePosy();
	  if(!(f.getStateLevel() == getCurrentLevel())) {
	    draw = false;
	  } 
	  int distance = 29;
	  double cx = (posx - posxx);
	  double cy = (posy - posyy);
	  double length = sqrt(cx*cx+cy*cy);
	  if(length != 0) {
	    posxx = (int)((cx/(length/distance)) + posxx);
	    posyy = (int)((cy/(length/distance)) + posyy);
	  }
	  if(draw) {
	      Transline* tl;
	      tl = new Transline(can); 
	      int distance = 29;
	      double cx = (posxx - posx);
	      double cy = (posyy - posy);
	      double length = sqrt(cx*cx+cy*cy);
	      if(length != 0) {
		int sx = (int)((cx/(length/distance)) + posx);
		int sy = (int)((cy/(length/distance)) + posy);
		tl->setPoints((int)posxx, (int)posyy, sx, sy);
		tl->setBrush(Qt::black);
		tl->id = a;
		placeItem(tl);
		drawArrow(posxx, posyy, posx, posy);
	      }
	  }
	}
      } 
    }
  }  
}

void View::drawTransPoints() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  bool hierarchy = false;
  int x;
  int y;
  int transid;
  int trackid;
  Q3CanvasItemList cil;
  if((f.getTransStart() < 0) && (f.getTransEnd() < 0)) {  
    if(f.getTransFirstTrackpoint()) {
      if(f.getTransTrackpointJoin()) {
	if(f.getTransTrackpointLevel() == getCurrentLevel()){
	  drawJoin();
	}
      } else if(f.getTransTrackpointLink()) {
	if(f.getTransTrackpointLevel() == getCurrentLevel()){
	  drawLinkentrance();  
	}
	while(f.getTransNextTrackpoint()) {}

      } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	if(f.getTransTrackpointLevel() == getCurrentLevel()){
	  drawHierarchy(); 
	  while(f.getTransNextTrackpoint()) {}
	} else {
	  hierarchy = true;
	}
      } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
	hierarchy = true;
      } else {
	if(f.getTransTrackpointLevel() == getCurrentLevel()){
	  drawTrackpoint();
	}
      }
      while(f.getTransNextTrackpoint()) {
	if(f.getTransTrackpointJoin()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){ 
	    transid = f.getTransID();
	    trackid = f.getTransTrackpointID();
	    int goal = f.getTransEnd();
	    bool drawjoin = true;
	    x = (int) f.getTransTrackpointPosx();
	    y = (int) f.getTransTrackpointPosy();
	    cil = canvas()->collisions(QPoint(x, y)); 
	    while (!cil.isEmpty()) {
	      Q3CanvasItem* i = cil.takeFirst();
	      if(i->rtti() == Joinpoint::RTTI) {
		f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
		if(f.getTransEnd() == goal) {
		  f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
		  double newtrackposx = f.getTransTrackpointPosx();
		  double newtrackposy = f.getTransTrackpointPosy();
		  f.getTransbyID(transid);
		  f.getTransTrackpointbyID(trackid);
		  f.modifyTrans(transid, trackid, newtrackposx, newtrackposy);
		  while(f.getTransNextTrackpoint()) {}
		  drawjoin = false;
		} else {
		  f.getTransbyID(transid);
		  f.getTransTrackpointbyID(trackid);
		}
	      }
	    }    
	    if(drawjoin) {
	      drawJoin();
	    }
	  }
	} else if(f.getTransTrackpointLink()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawLinkentrance();
	  }
	  while(f.getTransNextTrackpoint()) {}
	} else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawHierarchy();
	    while(f.getTransNextTrackpoint()) {}
	  } else {
	    hierarchy = true;
	  }
	} else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
	  hierarchy = true;
	} else {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    if(hierarchy) {
	      drawHierarchyexit();
	      hierarchy = false;
	    } else {
	      drawTrackpoint();
	    }
	  }
	}
      }
    } 
  } else {
    if(f.getTransStart() < 0) { 
      if(f.getTransFirstTrackpoint()) { 
	if(f.getTransTrackpointJoin()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawJoin();
	  }
	}  else if(f.getTransTrackpointLink()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawLinkentrance();  
	  }
	  while(f.getTransNextTrackpoint()) {}
	} else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawHierarchy();
	    while(f.getTransNextTrackpoint()) {}
	  } else {
	    hierarchy = true;
	  } 
	} else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
	  hierarchy = true;
	} else {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawTrackpoint();
	  }
	}
	while(f.getTransNextTrackpoint()) {
	   if(f.getTransTrackpointJoin()) {
	     if(f.getTransTrackpointLevel() == getCurrentLevel()){
	       transid = f.getTransID();
	       trackid = f.getTransTrackpointID();
	       int goal = f.getTransEnd();
	       bool drawjoin = true;
	       x = (int) f.getTransTrackpointPosx();
	       y = (int) f.getTransTrackpointPosy();
	       cil = canvas()->collisions(QPoint(x, y)); 
	       while (!cil.isEmpty()) {
		 Q3CanvasItem* i = cil.takeFirst();
		 if(i->rtti() == Joinpoint::RTTI) {
		   f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
		   if(f.getTransEnd() == goal) {
		     f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
		     double newtrackposx = f.getTransTrackpointPosx();
		     double newtrackposy = f.getTransTrackpointPosy();
		     f.getTransbyID(transid);
		     f.getTransTrackpointbyID(trackid);
		     f.modifyTrans(transid, trackid, newtrackposx, newtrackposy);
		     while(f.getTransNextTrackpoint()) {}
		     drawjoin = false;
		   } else {
		     f.getTransbyID(transid);
		     f.getTransTrackpointbyID(trackid);
		   }
		 }
	       } 
	       if(drawjoin) {
		 drawJoin();
	       }
	     }
	   } else if(f.getTransTrackpointLink()) {
	     if(f.getTransTrackpointLevel() == getCurrentLevel()){
	       drawLinkentrance();
	     }
	     while(f.getTransNextTrackpoint()) {}
	   } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	     if(f.getTransTrackpointLevel() == getCurrentLevel()){
	       drawHierarchy(); 
	       while(f.getTransNextTrackpoint()) {}
	     } else {
	       hierarchy = true;
	     }
	   } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
	     hierarchy = true;
	   } else {
	     if(f.getTransTrackpointLevel() == getCurrentLevel()){
	       if(hierarchy) {
		 drawHierarchyexit();
		 hierarchy = false;
	       } else {
		 drawTrackpoint();
	       }
	     }
	   }
	 }
      }
    } else {
      if(f.getTransFirstTrackpoint()) {
	if(f.getTransTrackpointJoin()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){ 
	    transid = f.getTransID();
	    trackid = f.getTransTrackpointID();
	    int goal = f.getTransEnd();
	    bool drawjoin = true;
	    x = (int) f.getTransTrackpointPosx();
	    y = (int) f.getTransTrackpointPosy();
	    cil = canvas()->collisions(QPoint(x, y)); 
	    while (!cil.isEmpty()) {
	      Q3CanvasItem* i = cil.takeFirst();
	      if(i->rtti() == Joinpoint::RTTI) {
		f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
		if(f.getTransEnd() == goal) {
		  f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
		  double newtrackposx = f.getTransTrackpointPosx();
		  double newtrackposy = f.getTransTrackpointPosy();
		  f.getTransbyID(transid);
		  f.getTransTrackpointbyID(trackid);
		  f.modifyTrans(transid, trackid, newtrackposx, newtrackposy);
		  while(f.getTransNextTrackpoint()) {}
		  drawjoin = false;
		} else {
		  f.getTransbyID(transid);
		  f.getTransTrackpointbyID(trackid);
		}
	      }
	    }  
	    if(drawjoin) {
	      drawJoin();
	    }
	  }
	} else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawHierarchy();
	    while(f.getTransNextTrackpoint()) {}
	  } else {
	    hierarchy = true;
	  }
	} else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
	  hierarchy = true;
	} else if(f.getTransTrackpointLink()) {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawLinkentrance(); 	
	  }
	  while(f.getTransNextTrackpoint()) {}
	} else {
	  if(f.getTransTrackpointLevel() == getCurrentLevel()){
	    drawTrackpoint();
	  }
	}
	while(f.getTransNextTrackpoint()) {
	  if(f.getTransTrackpointJoin()) {
	    if(f.getTransTrackpointLevel() == getCurrentLevel()){
	      transid = f.getTransID();
	      trackid = f.getTransTrackpointID();
	      int goal = f.getTransEnd();
	      bool drawjoin = true;
	      x = (int) f.getTransTrackpointPosx();
	      y = (int) f.getTransTrackpointPosy();
	      cil = canvas()->collisions(QPoint(x, y)); 
	      while (!cil.isEmpty()) {
		Q3CanvasItem* i = cil.takeFirst();
		if(i->rtti() == Joinpoint::RTTI) {
		  f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
		  if(f.getTransEnd() == goal) {
		    f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
		    double newtrackposx = f.getTransTrackpointPosx();
		    double newtrackposy = f.getTransTrackpointPosy();
		    f.getTransbyID(transid);
		    f.getTransTrackpointbyID(trackid);
		    f.modifyTrans(transid, trackid, newtrackposx, newtrackposy);
		    while(f.getTransNextTrackpoint()) {}
		    drawjoin = false;
		  } else {
		    f.getTransbyID(transid);
		    f.getTransTrackpointbyID(trackid);
		  }
		}
	      } 
	      if(drawjoin) {
		drawJoin();
	      }
	    }
	  } else if(f.getTransTrackpointLink()) {
	    if(f.getTransTrackpointLevel() == getCurrentLevel()){
	      drawLinkentrance(); 
	    }
	    while(f.getTransNextTrackpoint()) {}
	  } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
	    if(f.getTransTrackpointLevel() == getCurrentLevel()){
	      drawHierarchy();
	      while(f.getTransNextTrackpoint()) {}
	    } else {
	      hierarchy = true;
	    } 
	  } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
	    hierarchy = true;
	  } else {
	    if(f.getTransTrackpointLevel() == getCurrentLevel()){
	      if(hierarchy) {
		drawHierarchyexit();
		hierarchy = false;
	      } else {
		drawTrackpoint();
	      }
	    }
	  }
	}
      }
    }
  }  
}

void View::drawTrackpoint() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  TrackpointItem* tp; 
  tp = new TrackpointItem(can);
  tp->setBrush(Qt::black);
  tp->setSize(5,5); 
  int id =  f.getTransID();
  tp->transid = id;
  int trid = f.getTransTrackpointID();
  tp->id = trid;
  posx = f.getTransTrackpointPosx();
  posy = f.getTransTrackpointPosy();
  tp->move(posx - 2.5, posy - 2.5);
  placeItem(tp);
}

void View::drawJoin() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  Joinpoint* j;
  j = new Joinpoint(can);
  j->setBrush(Qt::red);
  j->setSize(10,10);
  int id =  f.getTransID();
  j->transid = id;
  int trid = f.getTransTrackpointID();
  j->id = trid;
  posx = f.getTransTrackpointPosx();
  posy = f.getTransTrackpointPosy();
  j->move(posx - 5, posy - 5);
  placeItem(j);
}

void View::drawLinkentrance() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  Linkentrance* le;
  le = new Linkentrance(can);
  le->setBrush(Qt::green);
  le->setSize(10,5);
  int id =  f.getTransID();
  le->transid = id;
  int trid = f.getTransTrackpointID();
  le->id = trid;
  posx = f.getTransTrackpointPosx();
  posy = f.getTransTrackpointPosy();
  le->move(posx, posy);
  placeItem(le); 
}

void View::drawHierarchyexit() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  Hierarchyexit* he;
  he = new Hierarchyexit(can);
  he->setBrush(Qt::blue);
  he->setSize(10,5);
  int id =  f.getTransID();
  he->transid = id;
  int trid = f.getTransTrackpointID();
  he->id = trid;
  posx = f.getTransTrackpointPosx();
  posy = f.getTransTrackpointPosy();
  he->move(posx, posy);
  placeItem(he);
}

void View::drawArrow(double ax, double ay, double bx, double by) {
  int distance = 29;
  int distance2 = 42;
  int arrowwidth = 8;
  double factor;
  double cx = (ax - bx);
  double cy = (ay - by);
  double length = sqrt(cx*cx+cy*cy);
  if(length != 0){
    int sx = (int)((cx/(length/distance)) + bx);
    int sy = (int)((cy/(length/distance)) + by);
    double dx = (cx/(length/distance2)) + bx;
    double dy = (cy/(length/distance2)) + by;
    double nx;
    double ny;
    if(cx == 0){
      nx = 1;
      ny = 0;
      factor = arrowwidth;
    } else if (cy == 0){
      ny = 1;
      nx = 0;
      factor = arrowwidth;
    } else {
      nx = -cy/cx;
      ny = 1;
      factor = sqrt(arrowwidth*arrowwidth/(nx*nx+ny*ny));
    }
    
    int e1x = (int)(dx + factor*nx);
    int e1y = (int)(dy + factor*ny);
    int e2x = (int)(dx + (-1)*factor*nx);
    int e2y = (int)(dy + (-1)*factor*ny);
    Transline* tl;
    tl = new Transline(can);
    tl->setPoints(sx, sy, e1x, e1y);
    tl->setBrush(Qt::black);
    tl->id = -1;
    placeItem(tl); 
    Transline* tl2;
    tl2 = new Transline(can);
    tl2->setPoints(sx, sy, e2x, e2y);
    tl2->setBrush(Qt::black);
    tl2->id = -1;
    placeItem(tl2);
  }
}

bool View::testJoin(int transid, int trackid) {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  testjoin = false;  
  f.getTransbyID(transid);
  if(f.getTransFirstTrackpoint()) {
    do {
      if(f.getTransTrackpointID() == trackid) {
	return testjoin;
      } else {
	if(f.getTransTrackpointJoin()) {
	  testjoin = true;
	}
      }
    } while(f.getTransNextTrackpoint());
  } 
  if(trackid < 1000){
    f.getTransTrackpointbyID(trackid);
  }
  return testjoin;
}

void View::refreshOtherJoins(int transid, int trackid) { 
  testjoin = false;
  int curhelp = -1;
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  f.getTransbyID(transid);
  f.getTransTrackpointbyID(trackid);
  if(f.getTransFirstTrackpoint()) {
    do {
      if(f.getTransTrackpointID() == trackid) {
	while (f.getTransNextTrackpoint()) {}
      } else {
	if(f.getTransTrackpointJoin()) {
	  curhelp = f.getTransTrackpointID();
	}
      }
    } while (f.getTransNextTrackpoint());
  }
  if(curhelp > -1) {
    trackid = curhelp;  
    f.getTransTrackpointbyID(trackid);
    int goal = f.getTransEnd();
    double jposx = f.getTransTrackpointPosx();
    double jposy = f.getTransTrackpointPosy();
    if(f.getFirstTrans()) {
      do {
	if (f.getTransEnd() == goal) {
	  if(f.getTransID() != transid){
	    if(f.getTransFirstTrackpoint()) {
	      do {
		if(f.getTransTrackpointJoin()) {
		  if((f.getTransTrackpointPosx() == jposx) && (f.getTransTrackpointPosy() == jposy)) {	  
		    int deleteid;
		    double curposx;
		    double curposy;
		    int curtransid = f.getTransID();
		    int curtrackid = f.getTransTrackpointID();
		    while(f.getTransNextTrackpoint()) {
		      deleteid = f.getTransTrackpointID();
		      f.deleteTrackpointOfTrans(curtransid, deleteid);
		      f.getTransTrackpointbyID(curtrackid);
		    }
		    f.getTransbyID(transid);
		    f.getTransTrackpointbyID(trackid);
		    while(f.getTransNextTrackpoint()) {
		      curposx = f.getTransTrackpointPosx();
		      curposy = f.getTransTrackpointPosy();
		      curhelp = f.getTransTrackpointLevel(); 
		      f.addTrackpointToTrans(curtransid, 1000, true, curposx, curposy, curhelp);
		      f.getTransbyID(curtransid);
		      curtrackid = f.getTransTrackpointID();
		      f.getTransbyID(transid);
		      curhelp = f.getTransTrackpointColor();
		      f.modifyTransTrackpointColor(curtransid, curtrackid, curhelp);
		      if(f.getTransTrackpointJoin()) {
			f.modifyTrans(curtransid, curtrackid, true);
		      }
		      if(f.getTransTrackpointLink()) {
			curhelp = f.getTransTrackpointLinkID();
			f.modifyTrans(curtransid, curtrackid, true, curhelp);
		      }
		      curhelp = f.getTransTrackpointHldirection();
		      f.transLevel(curtransid, curtrackid, curhelp);
		    }
		    f.getTransbyID(curtransid);
		  }
		}
	      } while (f.getTransNextTrackpoint());
	    }	
	  }
	}
      } while (f.getNextTrans());
    }
  }
}

void View::changeJoinTarget(int transid, int trackid, int target) {
  testjoin = false;
  int curhelp = -1;
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  f.getTransbyID(transid);
  f.getTransTrackpointbyID(trackid);
  if(f.getTransFirstTrackpoint()) {
    do {
      if(f.getTransTrackpointID() == trackid) {
	while (f.getTransNextTrackpoint()) {}
      } else {
	if(f.getTransTrackpointJoin()) {
	  curhelp = f.getTransTrackpointID();
	}
      }
    } while (f.getTransNextTrackpoint());
  }
  if(curhelp > -1) {
    trackid = curhelp;  
    f.getTransTrackpointbyID(trackid);
    int goal = f.getTransEnd();
    double jposx = f.getTransTrackpointPosx();
    double jposy = f.getTransTrackpointPosy();
    if(f.getFirstTrans()) {
      do {
	if (f.getTransEnd() == goal) {
	  if(f.getTransFirstTrackpoint()) {
	    do {
	      if(f.getTransTrackpointJoin()) {
		  if((f.getTransTrackpointPosx() == jposx) && (f.getTransTrackpointPosy() == jposy)) {	
		    int curid = f.getTransID();
		    int curstart = f.getTransStart();
		    f.modifyTrans(curid, curstart, target);
		    while (f.getTransNextTrackpoint()) {}
		  }
	      }
	    } while (f.getTransNextTrackpoint());
	  }	
	}
      } while (f.getNextTrans());
    } 
  }
}

void View::drawTransText() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  if(f.getTransLevel() == getCurrentLevel()) {  
    if(f.getTransStart() > -1 || f.getTransEnd() > -1 || (f.getTransFirstTrackpoint() && f.getTransNextTrackpoint())) {
      if(!(f.getTransFirstTrackpoint() && f.getTransTrackpointLink())) {
	TranslineText* tlt;
	tlt = new TranslineText(QString::fromStdString(f.getTransName()), can);
	tlt->id = f.getTransID();
	double px = f.getTransTextPosx();
	double py = f.getTransTextPosy();
	tlt->move(px, py);
	placeItem(tlt); 
      }
    }
  }
}

void View::resetposition() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  int id = dynamic_cast<TranslineText*>(activatedItem)->id;
  f.getTransbyID(id);
  int start = f.getTransStart();
  double px;
  double py; 
  if(start > -1) {
    f.getStatebyID(start);
    px = f.getStatePosx();
    py = f.getStatePosy(); 
    if(f.getTransFirstTrackpoint()) {
      px = f.getTransTrackpointPosx();
      py = f.getTransTrackpointPosy();
    } else {
      int distancetxt = 70;
      px = px + 15;
      py = py + 15;
      f.getStatebyID(f.getTransEnd());
      double gx = f.getStatePosx();
      double gy = f.getStatePosy();
      double cx = (gx - px-15);
      double cy = (gy - px-15);
      double length = sqrt(cx*cx+cy*cy); 
      if(length != 0) {
	double px = ((cx/(length/distancetxt)) + px);
	double py = ((cy/(length/distancetxt)) + py);
      }
      if(length > 80) {
	f.modifyTrans(helpvar , (double) px + 5, (double) py + 5, getCurrentLevel());
      } else {
	f.modifyTrans(helpvar , (double) px + 20, (double) py + 20, getCurrentLevel());
      }
    }
  } else {
    if(f.getTransFirstTrackpoint()) {
      px = f.getTransTrackpointPosx();
      py = f.getTransTrackpointPosy();
    } else {
      px = 10;
      py = 10;
    }
  }
  f.modifyTrans(id, px + 5, py + 5, getCurrentLevel());
  redraw();
}

void View::simulationMark(int type, int id) {
  activateItem(0);
  Q3CanvasItemList cil = canvas()->allItems();
  while (!cil.isEmpty()) {
    Q3CanvasItem* i = cil.takeFirst();
    switch(type) {
    case 1:	
      if(i->rtti() == StateItem::RTTI) {
	if((dynamic_cast<StateItem*>(i))->id == id) {
	  activateItem(i);
	  return;
	}
      }
      break;
    case 2: 
      if(i->rtti() == Transline::RTTI) {
	if((dynamic_cast<Transline*>(i))->id == id) {
	  activateItem(i);
	  return;
	}
      }
      break;
    case 3:
       if(i->rtti() == Hypertransition::RTTI) {
	if((dynamic_cast<Hypertransition*>(i))->id == id) {
	  activateItem(i);
	  return;
	}
       }
      break;
    default:

      break;
    }
  }

}


QColor View::getInitStateColor() {
  return statecolor;
}

void View::initStateColor(QColor c) {
  statecolor = c;
}
